@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C)      2022 Colin Ian King.
+ * Copyright (C) 2022-2025 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -18,6 +18,9 @@
  *
  */
 #include "stress-ng.h"
+#include "core-attribute.h"
+#include "core-builtin.h"
+#include "core-lock.h"
 #include "core-net.h"
 
 #if defined(HAVE_SYS_UN_H)
@@ -43,7 +46,9 @@ typedef struct {
 
 static const stress_domain_t domains[] = {
 	{ "ipv4",	AF_INET,	DOMAIN_INET },
+#if defined(AF_INET6)
 	{ "ipv6",	AF_INET6,	DOMAIN_INET6 },
+#endif
 	{ "unix",	AF_UNIX,	DOMAIN_UNIX },
 };
 
@@ -58,12 +63,12 @@ int stress_net_interface_exists(const char *interface, const int domain, struct
 	struct ifaddrs *ifaddr, *ifa;
 	int ret = -1;
 
-	if (!interface)
+	if (UNLIKELY(!interface))
 		return -1;
-	if (!addr)
+	if (UNLIKELY(!addr))
 		return -1;
 
-	if (getifaddrs(&ifaddr) < 0)
+	if (UNLIKELY(getifaddrs(&ifaddr) < 0))
 		return -1;
 	for (ifa = ifaddr; ifa; ifa = ifa->ifa_next) {
 		if (!ifa->ifa_addr)
@@ -73,7 +78,7 @@ int stress_net_interface_exists(const char *interface, const int domain, struct
 		if (ifa->ifa_addr->sa_family != domain)
 			continue;
 		if (strcmp(ifa->ifa_name, interface) == 0) {
-			(void)memcpy(addr, ifa->ifa_addr, sizeof(*addr));
+			(void)shim_memcpy(addr, ifa->ifa_addr, sizeof(*addr));
 			ret = 0;
 			break;
 		}
@@ -111,7 +116,7 @@ void stress_set_net_port(
  *  stress_net_domain()
  *	return human readable domain name from domain number
  */
-const char *stress_net_domain(const int domain)
+const char CONST *stress_net_domain(const int domain)
 {
 	size_t i;
 
@@ -174,7 +179,7 @@ int stress_set_sockaddr_if(
 	*len = 0;
 
 	/* omit ports 0..1023 */
-	if (sin_port < 1024)
+	if (UNLIKELY(sin_port < 1024))
 		sin_port += 1024;
 
 	switch (domain) {
@@ -182,7 +187,7 @@ int stress_set_sockaddr_if(
 	case AF_INET: {
 		static struct sockaddr_in addr;
 
-		(void)memset(&addr, 0, sizeof(addr));
+		(void)shim_memset(&addr, 0, sizeof(addr));
 
 		if ((!ifname) || (!stress_net_interface_exists(ifname, domain, (struct sockaddr *)&addr))) {
 			switch (net_addr) {
@@ -209,7 +214,7 @@ int stress_set_sockaddr_if(
 		static const struct in6_addr in6addr_any = IN6ADDR_ANY_INIT;
 		static const struct in6_addr in6addr_loopback = IN6ADDR_LOOPBACK_INIT;
 #endif
-		(void)memset(&addr, 0, sizeof(addr));
+		(void)shim_memset(&addr, 0, sizeof(addr));
 
 		if ((!ifname) || (!stress_net_interface_exists(ifname, domain, (struct sockaddr *)&addr))) {
 			switch (net_addr) {
@@ -234,10 +239,10 @@ int stress_set_sockaddr_if(
 	case AF_UNIX: {
 		static struct sockaddr_un addr;
 
-		(void)memset(&addr, 0, sizeof(addr));
+		(void)shim_memset(&addr, 0, sizeof(addr));
 		addr.sun_family = AF_UNIX;
 		(void)snprintf(addr.sun_path, sizeof(addr.sun_path),
-			"/tmp/stress-ng-%jd-%" PRIu32,
+			"/tmp/stress-ng-%" PRIdMAX "-%" PRIu32,
 			(intmax_t)pid, instance);
 		*sockaddr = (struct sockaddr *)&addr;
 		*len = sizeof(addr);
@@ -272,7 +277,7 @@ int stress_set_sockaddr(
  *  stress_set_sockaddr_port()
  *	setup just the socket address port
  */
-void HOT stress_set_sockaddr_port(
+void stress_set_sockaddr_port(
 	const int domain,
 	const int port,
 	struct sockaddr *sockaddr)
@@ -307,13 +312,13 @@ void HOT stress_set_sockaddr_port(
  *  stress_net_port_range_ok()
  *	port range sanity check, returns true if OK
  */
-static bool stress_net_port_range_ok(const int start_port, const int end_port)
+static bool CONST stress_net_port_range_ok(const int start_port, const int end_port)
 {
-	if ((start_port > end_port))
+	if (UNLIKELY((start_port > end_port)))
 		return false;
-	if ((start_port < 0) || (start_port >= 65536))
+	if (UNLIKELY((start_port < 0) || (start_port >= 65536)))
 		return false;
-	if ((end_port < 0) || (end_port >= 65536))
+	if (UNLIKELY((end_port < 0) || (end_port >= 65536)))
 		return false;
 	return true;
 }
@@ -321,16 +326,16 @@ static bool stress_net_port_range_ok(const int start_port, const int end_port)
 /*
  *   stress_net_reserve_ports()
  *	attempt to reserve ports, returns nearest available contiguous
- *	ports that are availble or -1 if none could be found
+ *	ports that are available or -1 if none could be found
  */
 int stress_net_reserve_ports(const int start_port, const int end_port)
 {
 	int i, port = -1;
 
-	if (!stress_net_port_range_ok(start_port, end_port))
+	if (UNLIKELY(!stress_net_port_range_ok(start_port, end_port)))
 		return -1;
 
-	if (stress_lock_acquire(g_shared->net_port_map.lock) < 0)
+	if (UNLIKELY(stress_lock_acquire(g_shared->net_port_map.lock) < 0))
 		return -1;
 
 	if (LIKELY(start_port == end_port)) {
@@ -375,10 +380,35 @@ int stress_net_reserve_ports(const int start_port, const int end_port)
  */
 void stress_net_release_ports(const int start_port, const int end_port)
 {
-	if (stress_net_port_range_ok(start_port, end_port)) {
+	if (LIKELY(stress_net_port_range_ok(start_port, end_port))) {
 		int i;
 
 		for (i = start_port; i <= end_port; i++)
 			STRESS_CLRBIT(g_shared->net_port_map.allocated, i);
 	}
 }
+
+/*
+ *  stress_ipv4_checksum()
+ *	ipv4 data checksum
+ */
+uint16_t CONST OPTIMIZE3 stress_ipv4_checksum(uint16_t *ptr, const size_t sz)
+{
+	register uint32_t sum = 0;
+	register size_t n = sz;
+
+	if (UNLIKELY(!ptr))
+		return 0;
+
+	while (n > 1) {
+		sum += *ptr++;
+		n -= 2;
+	}
+
+	if (n)
+		sum += *(uint8_t*)ptr;
+	sum = (sum >> 16) + (sum & 0xffff);
+	sum += (sum >> 16);
+
+	return (uint16_t)~sum;
+}
