@@ -89,6 +89,7 @@
 #define OPT_FLAGS_STRESSOR_TIME	 STRESS_BIT_ULL(58)	/* --stressor-time */
 #define OPT_FLAGS_TASKSET_RANDOM STRESS_BIT_ULL(59)	/* --taskset-random */
 #define OPT_FLAGS_BUILDINFO	 STRESS_BIT_ULL(60)	/* --buildinfo */
+#define OPT_FLAGS_AUTOGROUP	 STRESS_BIT_ULL(61)	/* --autogroup */
 
 #define OPT_FLAGS_MINMAX_MASK		\
 	(OPT_FLAGS_MINIMIZE | OPT_FLAGS_MAXIMIZE)
@@ -107,6 +108,7 @@ typedef enum {
 	OPT_undefined = 0,
 	/* Short options */
 	OPT_query = '?',
+	OPT_aggressive = 'A',
 	OPT_all = 'a',
 	OPT_backoff = 'b',
 	OPT_bigheap = 'B',
@@ -118,14 +120,17 @@ typedef enum {
 	OPT_fallocate = 'F',
 	OPT_help = 'h',
 	OPT_io = 'i',
+	OPT_iostat = 'I',
 	OPT_job = 'j',
 	OPT_keep_name = 'k',
 	OPT_klog_check = 'K',
 	OPT_cpu_load = 'l',
+	OPT_log_file = 'L',
 	OPT_vm = 'm',
 	OPT_metrics = 'M',
 	OPT_dry_run = 'n',
 	OPT_open = 'o',
+	OPT_oomable = 'O',
 	OPT_pipe = 'p',
 	OPT_poll = 'P',
 	OPT_quiet = 'q',
@@ -167,8 +172,6 @@ typedef enum {
 	OPT_af_alg_ops,
 	OPT_af_alg_dump,
 
-	OPT_aggressive,
-
 	OPT_aio,
 	OPT_aio_ops,
 	OPT_aio_requests,
@@ -186,6 +189,8 @@ typedef enum {
 	OPT_atomic,
 	OPT_atomic_ops,
 
+	OPT_autogroup,
+
 	OPT_bad_altstack,
 	OPT_bad_altstack_ops,
 
@@ -662,8 +667,6 @@ typedef enum {
 	OPT_ioprio,
 	OPT_ioprio_ops,
 
-	OPT_iostat,
-
 	OPT_io_ops,
 
 	OPT_io_uring,
@@ -773,7 +776,6 @@ typedef enum {
 	OPT_lockofd_ops,
 
 	OPT_log_brief,
-	OPT_log_file,
 	OPT_log_lockless,
 
 	OPT_logmath,
@@ -1056,7 +1058,6 @@ typedef enum {
 	OPT_numa_shuffle_addr,
 	OPT_numa_shuffle_node,
 
-	OPT_oomable,
 	OPT_oom_avoid,
 	OPT_oom_avoid_bytes,
 
@@ -1609,6 +1610,9 @@ typedef enum {
 	OPT_timerfd_freq,
 	OPT_timerfd_rand,
 
+	OPT_timermix,
+	OPT_timermix_ops,
+
 	OPT_times,
 
 	OPT_timestamp,
