@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2023-2024 Colin Ian King.
+ * Copyright (C) 2023-2025 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -19,6 +19,7 @@
 #include "stress-ng.h"
 #include "core-lock.h"
 #include "core-madvise.h"
+#include "core-mmap.h"
 #include "core-stressors.h"
 #include "core-shared-heap.h"
 
@@ -29,9 +30,9 @@
  *   a shared memory heap.
  */
 #if defined(HAVE_BUILTIN_CONSTANT_P)
-#define STRESS_MAX_SHARED_HEAP_SIZE		(16 * KB)
-#else
 #define STRESS_MAX_SHARED_HEAP_SIZE		(64 * KB)
+#else
+#define STRESS_MAX_SHARED_HEAP_SIZE		(256 * KB)
 #endif
 
 /* Used just to determine number of stressors via STRESS_MAX */
@@ -53,24 +54,23 @@ void *stress_shared_heap_init(void)
 {
 	const size_t page_size = stress_get_page_size();
 
-	/* Allocate enough heap for all stressor descriptions with 50% metrics allocated */
-	size_t size = (STRESS_MISC_METRICS_MAX * (32 + sizeof(void *)) * STRESS_MAX) / 2;
+	/* Allocate enough heap for all stressor descriptions with 100% metrics allocated */
+	size_t size = (STRESS_MISC_METRICS_MAX * (32 + sizeof(void *)) * STRESS_MAX);
 
 	size = STRESS_MINIMUM(size, STRESS_MAX_SHARED_HEAP_SIZE);
 	g_shared->shared_heap.out_of_memory = false;
 	g_shared->shared_heap.heap_size = (size + page_size - 1) & ~(page_size - 1);
 	g_shared->shared_heap.str_list_head = NULL;
-	g_shared->shared_heap.heap = stress_mmap_populate(NULL, size,
-					PROT_READ | PROT_WRITE,
-					MAP_ANONYMOUS | MAP_SHARED, -1, 0);
-	if (g_shared->shared_heap.heap == MAP_FAILED) {
+	g_shared->shared_heap.heap = stress_mmap_anon_shared(size, PROT_READ | PROT_WRITE);
+	if (UNLIKELY(g_shared->shared_heap.heap == MAP_FAILED)) {
 		g_shared->shared_heap.lock = NULL;
 		return NULL;
 	}
+	stress_set_vma_anon_name(g_shared->shared_heap.heap, size, "shared-heap");
 	(void)stress_madvise_mergeable(g_shared->shared_heap.heap, size);
-	g_shared->shared_heap.lock = stress_lock_create();
-	if (!g_shared->shared_heap.lock) {
-		(void)munmap((void *)g_shared->shared_heap.heap, g_shared->shared_heap.heap_size);
+	g_shared->shared_heap.lock = stress_lock_create("shared-heap");
+	if (UNLIKELY(!g_shared->shared_heap.lock)) {
+		(void)stress_munmap_anon_shared((void *)g_shared->shared_heap.heap, g_shared->shared_heap.heap_size);
 		g_shared->shared_heap.heap = NULL;
 		return NULL;
 	}
@@ -78,10 +78,10 @@ void *stress_shared_heap_init(void)
 }
 
 /*
- *  stress_shared_heap_deinit()
+ *  stress_shared_heap_free()
  *	free shared heap
  */
-void stress_shared_heap_deinit(void)
+void stress_shared_heap_free(void)
 {
 	if (g_shared->shared_heap.out_of_memory) {
 		pr_inf("shared heap: out of memory duplicating some strings, increase STRESS_MAX_SHARED_HEAP_SIZE to fix this\n");
@@ -92,7 +92,7 @@ void stress_shared_heap_deinit(void)
 	}
 #endif
 	if (g_shared->shared_heap.heap) {
-		(void)munmap((void *)g_shared->shared_heap.heap, g_shared->shared_heap.heap_size);
+		(void)stress_munmap_anon_shared((void *)g_shared->shared_heap.heap, g_shared->shared_heap.heap_size);
 		g_shared->shared_heap.heap = NULL;
 	}
 	if (g_shared->shared_heap.lock) {
@@ -113,7 +113,7 @@ void *stress_shared_heap_malloc(const size_t size)
 	ssize_t heap_free;
 	void *ptr;
 
-	if (stress_lock_acquire(g_shared->shared_heap.lock) < 0)
+	if (UNLIKELY(stress_lock_acquire(g_shared->shared_heap.lock) < 0))
 		return NULL;
 
 	heap_free = g_shared->shared_heap.heap_size - g_shared->shared_heap.offset;
@@ -132,7 +132,7 @@ void *stress_shared_heap_malloc(const size_t size)
 /*
  *  stress_shared_heap_dup_const()
  *	Do a strdup of string using the shared heap. String must never be
- *	modified as this dup operation re-used existing identical strings
+ *	modified as this dup operation reused existing identical strings
  *	allocated on the shared heap. This is designed for storing metric
  *	descriptions that get allocated per stressor and we want to reduce
  *	duplicated allocations where possible.
@@ -142,7 +142,7 @@ char *stress_shared_heap_dup_const(const char *str)
 	size_t len, str_len;
 	stress_shared_heap_str_t *heap_str;
 
-	if (stress_lock_acquire(g_shared->shared_heap.lock) < 0)
+	if (UNLIKELY(stress_lock_acquire(g_shared->shared_heap.lock) < 0))
 		return NULL;
 
 	for (heap_str = (stress_shared_heap_str_t *)g_shared->shared_heap.str_list_head; heap_str; heap_str = heap_str->next) {
@@ -155,21 +155,21 @@ char *stress_shared_heap_dup_const(const char *str)
 	str_len = strlen(str) + 1;
 	len = str_len + sizeof(void *);
 	heap_str = (stress_shared_heap_str_t *)stress_shared_heap_malloc(len);
-	if (!heap_str)
+	if (UNLIKELY(!heap_str))
 		return NULL;
 
-	(void)shim_strlcpy(heap_str->str, str, str_len);
+	(void)shim_strscpy(heap_str->str, str, str_len);
 	heap_str->next = NULL;
 
 	/*
 	 *  We failed to acquire so we can't add to list, return dup'd string
 	 *  and skip adding it to the list, at least the dup worked!
 	 */
-	if (stress_lock_acquire(g_shared->shared_heap.lock) < 0)
+	if (UNLIKELY(stress_lock_acquire(g_shared->shared_heap.lock) < 0))
 		return heap_str->str;
 
 	/*
-	 *  Save a copy so it can be re-used
+	 *  Save a copy so it can be reused
 	 */
 	heap_str->next = (stress_shared_heap_str_t *)g_shared->shared_heap.str_list_head;
 	g_shared->shared_heap.str_list_head = (void *)heap_str;
