@@ -205,7 +205,7 @@ PRAGMA_UNROLL_N(2)
  *  stress_aiol_check_buffer()
  *	check buffer contains some known pattern
  */
-static inline PURE OPTIMIZE3 bool stress_aiol_check_buffer(
+static inline CONST OPTIMIZE3 bool stress_aiol_check_buffer(
 	const uint8_t pattern,
 	const uint8_t *const buffer,
 	const size_t size)
@@ -354,8 +354,8 @@ free_cb:
 free_buffer:
 	free(info->buffer);
 err_msg:
-	pr_inf_skip("%s: out of memory allocating buffers, skipping stressors\n",
-		args->name);
+	pr_inf_skip("%s: out of memory allocating buffers%s, skipping stressors\n",
+		args->name, stress_get_memfree_str());
 
 	(void)shim_memset(info, 0, sizeof(*info));
 	return -1;
@@ -427,7 +427,7 @@ static int stress_aiol(stress_args_t *args)
 		aio_max_nr = 1;
 	if (aiol_requests > aio_max_nr) {
 		aiol_requests = aio_max_nr;
-		if (args->instance == 0)
+		if (stress_instance_zero(args))
 			pr_inf("%s: Limiting AIO requests to "
 				"%" PRIu32 " per stressor (avoids running out of resources)\n",
 				args->name, aiol_requests);
@@ -852,7 +852,7 @@ free_memory:
 
 const stressor_info_t stress_aiol_info = {
 	.stressor = stress_aiol,
-	.class = CLASS_IO | CLASS_INTERRUPT | CLASS_OS,
+	.classifier = CLASS_IO | CLASS_INTERRUPT | CLASS_OS,
 	.opts = opts,
 	.verify = VERIFY_ALWAYS,
 	.help = help
@@ -860,7 +860,7 @@ const stressor_info_t stress_aiol_info = {
 #else
 const stressor_info_t stress_aiol_info = {
 	.stressor = stress_unimplemented,
-	.class = CLASS_IO | CLASS_INTERRUPT | CLASS_OS,
+	.classifier = CLASS_IO | CLASS_INTERRUPT | CLASS_OS,
 	.opts = opts,
 	.help = help,
 	.verify = VERIFY_ALWAYS,
