@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C)      2022 Colin Ian King.
+ * Copyright (C) 2022-2025 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -18,8 +18,12 @@
  *
  */
 #include "stress-ng.h"
+#include "core-attribute.h"
+#include "core-builtin.h"
 #include "core-capabilities.h"
 
+#include <time.h>
+
 #if defined(HAVE_POLL_H)
 #include <poll.h>
 #endif
@@ -34,14 +38,6 @@ static const stress_help_t help[] = {
 	{ NULL,	NULL,		NULL }
 };
 
-#if defined(HAVE_SYS_TIMEX_H)
-#define shim_timex      timex
-#else
-struct shim_timex {
-	char data[128];
-};
-#endif
-
 #if defined(HAVE_LIB_RT) &&		\
     defined(HAVE_CLOCK_GETTIME) &&	\
     defined(HAVE_CLOCK_SETTIME)
@@ -84,6 +80,9 @@ static const stress_clock_info_t clocks[] = {
 #if defined(CLOCK_TAI)
 	CLOCK_INFO(CLOCK_TAI)
 #endif
+#if defined(CLOCK_AUX)
+	CLOCK_INFO(CLOCK_AUX)
+#endif
 };
 
 #if defined(HAVE_CLOCK_NANOSLEEP)
@@ -118,6 +117,8 @@ static const int timers[] = {
 };
 #endif
 
+#define MAX_TIMERS	(SIZEOF_ARRAY(timers))
+
 #if defined(HAVE_CLOCK_NANOSLEEP) || 	\
     (defined(HAVE_TIMER_CREATE) &&	\
     defined(HAVE_TIMER_DELETE) &&	\
@@ -128,7 +129,7 @@ static const int timers[] = {
  *  stress_clock_name()
  *	clock id to name
  */
-static const char *stress_clock_name(int id)
+static const char * PURE stress_clock_name(int id)
 {
 	size_t i;
 
@@ -156,36 +157,22 @@ static const char *stress_clock_name(int id)
 static inline bool check_invalid_clock_id(const clockid_t id) {
         struct timespec tp;
 
-        (void)memset(&tp, 0, sizeof(tp));
+        (void)shim_memset(&tp, 0, sizeof(tp));
         return (clock_gettime(id, &tp) != 0);
 }
 #endif
 
 #define FD_TO_CLOCKID(fd)	((~(clockid_t)(fd) << 3) | 3)
 
-#if defined(__NR_clock_adjtime) &&	\
-    defined(HAVE_SYSCALL) &&		\
-    defined(HAVE_SYS_TIMEX_H) &&	\
-    defined(CLOCK_THREAD_CPUTIME_ID) &&	\
-    defined(ADJ_SETOFFSET)
-/*
- *   shim_clock_adjtime
- *	wrapper for linux clock_adjtime system call
- */
-static int shim_clock_adjtime(clockid_t clk_id, struct shim_timex *tx)
-{
-	return (int)syscall(__NR_clock_adjtime, clk_id, tx);
-}
-#endif
-
 /*
  *  stress_clock()
  *	stress system by rapid clocking system calls
  */
-static int stress_clock(const stress_args_t *args)
+static int stress_clock(stress_args_t *args)
 {
 	bool test_invalid_timespec = true;
 	const bool is_root = stress_check_capability(SHIM_CAP_IS_ROOT);
+	int rc = EXIT_SUCCESS;
 
 #if defined(CHECK_INVALID_CLOCK_ID)
 	const bool invalid_clock_id = check_invalid_clock_id(INT_MAX);
@@ -197,6 +184,8 @@ static int stress_clock(const stress_args_t *args)
 	 */
 	stress_mwc_set_seed(0xf238, 0x1872);
 
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	do {
@@ -211,10 +200,14 @@ static int stress_clock(const stress_args_t *args)
 			 *  Exercise setting local thread CPU timer
 			 */
 			ret = shim_clock_gettime(CLOCK_THREAD_CPUTIME_ID, &t);
-			if ((ret < 0) && (g_opt_flags & OPT_FLAGS_VERIFY) &&
-			    (errno != EINVAL) && (errno != ENOSYS))
+			if (UNLIKELY((ret < 0) && (g_opt_flags & OPT_FLAGS_VERIFY) &&
+				     (errno != EINVAL) &&
+				     (errno != ENOSYS) &&
+				     (errno != EINTR))) {
 				pr_fail("%s: clock_gettime failed for timer 'CLOCK_THREAD_CPUTIME_ID', errno=%d (%s)\n",
 					args->name, errno, strerror(errno));
+				rc = EXIT_FAILURE;
+			}
 
 			/* Exercise clock_settime with illegal clockid */
 			(void)shim_clock_settime((clockid_t)-1, &t);
@@ -250,15 +243,21 @@ static int stress_clock(const stress_args_t *args)
 				int ret;
 
 				ret = shim_clock_getres(clocks[i].id, &t);
-				if ((ret < 0) && (g_opt_flags & OPT_FLAGS_VERIFY) &&
-			            (errno != EINVAL) && (errno != ENOSYS))
+				if (UNLIKELY((ret < 0) && (g_opt_flags & OPT_FLAGS_VERIFY) &&
+					     (errno != EINVAL) &&
+					     (errno != ENOSYS))) {
 					pr_fail("%s: clock_getres failed for timer '%s', errno=%d (%s)\n",
 							args->name, clocks[i].name, errno, strerror(errno));
+					rc = EXIT_FAILURE;
+				}
 				ret = shim_clock_gettime(clocks[i].id, &t);
-				if ((ret < 0) && (g_opt_flags & OPT_FLAGS_VERIFY) &&
-			            (errno != EINVAL) && (errno != ENOSYS))
+				if (UNLIKELY((ret < 0) && (g_opt_flags & OPT_FLAGS_VERIFY) &&
+					     (errno != EINVAL) &&
+					     (errno != ENOSYS))) {
 					pr_fail("%s: clock_gettime failed for timer '%s', errno=%d (%s)\n",
 						args->name, clocks[i].name, errno, strerror(errno));
+					rc = EXIT_FAILURE;
+				}
 			}
 		}
 #else
@@ -276,22 +275,25 @@ static int stress_clock(const stress_args_t *args)
 
 				/* Save current time to reset later if required */
 				ret = shim_clock_gettime(clocks[i].id, &t1);
-				if ((ret < 0) && (g_opt_flags & OPT_FLAGS_VERIFY) &&
-				    (errno != EINVAL) && (errno != ENOSYS)) {
+				if (UNLIKELY((ret < 0) && (g_opt_flags & OPT_FLAGS_VERIFY) &&
+					     (errno != EINVAL) &&
+					     (errno != ENOSYS))) {
 					pr_fail("%s: clock_getres failed for timer '%s', errno=%d (%s)\n",
-					args->name, clocks[i].name, errno, strerror(errno));
+						args->name, clocks[i].name, errno, strerror(errno));
+					rc = EXIT_FAILURE;
 				}
-				if (ret < 0)
+				if (UNLIKELY(ret < 0))
 					continue;
 
 				/* Ensuring clock_settime cannot succeed without privilege */
 				if (!is_root) {
 					ret = shim_clock_settime(clocks[i].id, &t1);
-					if (ret != -EPERM) {
+					if (UNLIKELY(ret != -EPERM)) {
 						/* This is an error, report it! */
 						pr_fail("%s: clock_settime failed, did not have privilege to "
 							"set time, expected -EPERM, instead got errno=%d (%s)\n",
 							args->name, errno, strerror(errno));
+						rc = EXIT_FAILURE;
 					}
 				}
 
@@ -310,19 +312,21 @@ static int stress_clock(const stress_args_t *args)
 				 */
 				if ((t.tv_sec < 0) && (t.tv_nsec < 0)) {	/* cppcheck-suppress knownConditionTrueFalse */
 					ret = shim_clock_settime(clocks[i].id, &t);
-					if (ret < 0)
+					if (UNLIKELY(ret < 0))
 						continue;
 
 					/* Expected a failure, but it succeeded(!) */
 					pr_fail("%s: clock_settime was able to set an "
 						"invalid negative time for timer '%s'\n",
 						args->name, clocks[i].name);
+					rc = EXIT_FAILURE;
 
 					/* Restore the correct time */
 					ret = shim_clock_settime(clocks[i].id, &t1);
-					if ((ret < 0) && (errno != EINVAL) && (errno != ENOSYS)) {
+					if (UNLIKELY((ret < 0) && (errno != EINVAL) && (errno != ENOSYS))) {
 						pr_fail("%s: clock_gettime failed for timer '%s', errno=%d (%s)\n",
 							args->name, clocks[i].name, errno, strerror(errno));
+						rc = EXIT_FAILURE;
 					}
 					/*
 					 * Ensuring invalid clock_settime runs
@@ -348,15 +352,15 @@ static int stress_clock(const stress_args_t *args)
 
 				/* Exercise clock_nanosleep on invalid clock id */
 				if (invalid_clock_id) {
-					(void)memset(&t, 0, sizeof(t));
+					(void)shim_memset(&t, 0, sizeof(t));
 					VOID_RET(int, clock_nanosleep(INT_MAX, TIMER_ABSTIME, &t, NULL));
 				}
 
-				(void)memset(&t, 0, sizeof(t));
+				(void)shim_memset(&t, 0, sizeof(t));
 				t.tv_sec = -1;
 				VOID_RET(int, clock_nanosleep(clocks_nanosleep[0], TIMER_ABSTIME, &t, NULL));
 
-				(void)memset(&t, 0, sizeof(t));
+				(void)shim_memset(&t, 0, sizeof(t));
 				t.tv_nsec = STRESS_NANOSECOND;
 				VOID_RET(int, clock_nanosleep(clocks_nanosleep[0], TIMER_ABSTIME, &t, NULL));
 			}
@@ -368,17 +372,21 @@ static int stress_clock(const stress_args_t *args)
 				int ret;
 
 				t.tv_sec = 0;
-				t.tv_nsec = (stress_mwc32() % 2500) + 1;
+				t.tv_nsec = stress_mwc32modn(2500) + 1;
 				/*
 				 *  Calling with TIMER_ABSTIME will force
 				 *  clock_nanosleep() to return immediately
 				 */
 				ret = clock_nanosleep(clocks_nanosleep[i], TIMER_ABSTIME, &t, NULL);
-				if ((ret < 0) && (g_opt_flags & OPT_FLAGS_VERIFY))
-					pr_fail("%s: clock_nanosleep failed for timer '%s', errno=%d (%s)\n",
-						args->name,
-						stress_clock_name(clocks_nanosleep[i]),
-						errno, strerror(errno));
+				if (UNLIKELY((ret < 0) && (g_opt_flags & OPT_FLAGS_VERIFY))) {
+					if (errno != EINTR) {
+						pr_fail("%s: clock_nanosleep failed for timer '%s', errno=%d (%s)\n",
+							args->name,
+							stress_clock_name(clocks_nanosleep[i]),
+							errno, strerror(errno));
+						rc = EXIT_FAILURE;
+					}
+				}
 			}
 		}
 #else
@@ -386,37 +394,38 @@ static int stress_clock(const stress_args_t *args)
 #endif
 
 #if defined(__NR_clock_adjtime) &&	\
+    defined(HAVE_TIMEX) &&		\
     defined(HAVE_SYS_TIMEX_H) &&	\
     defined(CLOCK_THREAD_CPUTIME_ID) &&	\
     defined(ADJ_SETOFFSET)
 		{
 			size_t i;
-			struct shim_timex tx;
+			shim_timex_t tx;
 
 			/* Exercise clock_adjtime on invalid clock id */
 			if (invalid_clock_id) {
-				(void)memset(&tx, 0, sizeof(tx));
+				(void)shim_memset(&tx, 0, sizeof(tx));
 				VOID_RET(int, shim_clock_adjtime(INT_MAX, &tx));
 			}
 
+			(void)shim_memset(&tx, 0, sizeof(tx));
 			/*
 			 *  Exercise clock_adjtime
 			 */
 			for (i = 0; i < SIZEOF_ARRAY(clocks); i++) {
 				int ret;
 
-				(void)memset(&tx, 0, sizeof(tx));
-
 				tx.modes = ADJ_SETOFFSET;
 				tx.time.tv_sec = 0;
 				tx.time.tv_usec = 0;
 
 				ret = shim_clock_adjtime(clocks[i].id, &tx);
-				if ((ret < 0) && (g_opt_flags & OPT_FLAGS_VERIFY) &&
-			            (errno != EINVAL) && (errno != ENOSYS) &&
-				    (errno != EPERM) && (errno != EOPNOTSUPP)) {
+				if (UNLIKELY(((ret < 0) && (g_opt_flags & OPT_FLAGS_VERIFY) &&
+					     (errno != EINVAL) && (errno != ENOSYS) &&
+					     (errno != EPERM) && (errno != EOPNOTSUPP)))) {
 					pr_fail("%s: clock_adjtime failed for timer '%s', errno=%d (%s)\n",
 						args->name, clocks[i].name, errno, strerror(errno));
+					rc = EXIT_FAILURE;
 				}
 			}
 		}
@@ -432,77 +441,78 @@ static int stress_clock(const stress_args_t *args)
     defined(HAVE_TIMER_SETTIME)
 		{
 			size_t i;
-			const size_t n = SIZEOF_ARRAY(timers);
-			bool timer_fail[n];
-			timer_t timer_id[n];
+			bool timer_fail[MAX_TIMERS];
+			timer_t timer_id[MAX_TIMERS];
 			struct itimerspec its;
+			struct sigevent sevp;
 			int ret;
 
+			(void)shim_memset(&sevp, 0, sizeof(sevp));
 			/*
 			 *  Stress the timers
 			 */
-			for (i = 0; i < n; i++) {
-				struct sigevent sevp;
-
+			for (i = 0; i < MAX_TIMERS; i++) {
 				timer_fail[i] = false;
 				timer_id[i] = (timer_t)-1;
 
-				(void)memset(&sevp, 0, sizeof(sevp));
 				sevp.sigev_notify = SIGEV_NONE;
 				ret = timer_create(timers[i], &sevp, &timer_id[i]);
-				if (ret < 0) {
+				if (UNLIKELY(ret < 0)) {
 					timer_fail[i] = true;
 					if (g_opt_flags & OPT_FLAGS_VERIFY) {
-						if ((errno == EINVAL) || (errno == EPERM))
+						if ((errno == EINVAL) || (errno == EPERM) || (errno == ENOTSUP))
 							continue;
 						pr_fail("%s: timer_create failed for timer '%s', errno=%d (%s)\n",
 							args->name,
 							stress_clock_name(timers[i]),
 							errno, strerror(errno));
+						rc = EXIT_FAILURE;
 					}
 					continue;
 				}
 
 				/* One shot mode, for random time 0..5000 ns */
 				its.it_value.tv_sec = 0;
-				its.it_value.tv_nsec = (stress_mwc32() % 5000) + 1;
+				its.it_value.tv_nsec = stress_mwc32modn(5000) + 1;
 				its.it_interval.tv_sec = 0;
 				its.it_interval.tv_nsec = 0;
 
 				ret = timer_settime(timer_id[i], 0, &its, NULL);
-				if ((ret < 0) && (g_opt_flags & OPT_FLAGS_VERIFY)) {
+				if (UNLIKELY((ret < 0) && (g_opt_flags & OPT_FLAGS_VERIFY))) {
 					pr_fail("%s: timer_settime failed for timer '%s', errno=%d (%s)\n",
 						args->name,
 						stress_clock_name(timers[i]),
 						errno, strerror(errno));
+					rc = EXIT_FAILURE;
 				}
 			}
 
-			for (i = 0; i < n; i++) {
-				if (timer_fail[i] || (timer_id[i] == (timer_t)-1))
+			for (i = 0; i < MAX_TIMERS; i++) {
+				if (UNLIKELY(timer_fail[i] || (timer_id[i] == (timer_t)-1)))
 					continue;
 
 				ret = timer_gettime(timer_id[i], &its);
-				if ((ret < 0) && (g_opt_flags & OPT_FLAGS_VERIFY)) {
+				if (UNLIKELY((ret < 0) && (g_opt_flags & OPT_FLAGS_VERIFY))) {
 					pr_fail("%s: timer_gettime failed for timer '%s', errno=%d (%s)\n",
 						args->name,
 						stress_clock_name(timers[i]),
 						errno, strerror(errno));
+					rc = EXIT_FAILURE;
 					break;
 				}
-				ret = timer_getoverrun(timer_id[i]);
-				(void)ret;
+				VOID_RET(int, timer_getoverrun(timer_id[i]));
 			}
 
-			for (i = 0; i < n; i++) {
-				if (timer_fail[i] || (timer_id[i] == (timer_t)-1))
+			for (i = 0; i < MAX_TIMERS; i++) {
+				if (UNLIKELY(timer_fail[i] || (timer_id[i] == (timer_t)-1)))
 					continue;
 				ret = timer_delete(timer_id[i]);
-				if ((ret < 0) && (g_opt_flags & OPT_FLAGS_VERIFY)) {
+				if (UNLIKELY((ret < 0) && (g_opt_flags & OPT_FLAGS_VERIFY))) {
 					pr_fail("%s: timer_delete failed for timer '%s', errno=%d (%s)\n",
 						args->name,
 						stress_clock_name(timers[i]),
 						errno, strerror(errno));
+					rc = EXIT_FAILURE;
 					break;
 				}
 			}
@@ -518,16 +528,17 @@ static int stress_clock(const stress_args_t *args)
 			fd = open("/dev/ptp0", O_RDWR);
 			if (fd >= 0) {
 				struct timespec t;
-				int ret, clkid = FD_TO_CLOCKID(fd);
-#if defined(HAVE_POLL_H)
+				int ret;
+				const int clkid = FD_TO_CLOCKID(fd);
+#if defined(HAVE_POLL_H) &&	\
+    defined(HAVE_POLL)
 				struct pollfd pollfds[1];
 
 				pollfds[0].fd = fd;
 				pollfds[0].events = POLLIN;
 				pollfds[0].revents = 0;
 
-				ret = poll(pollfds, 1, 0);
-				(void)ret;
+				VOID_RET(int, poll(pollfds, 1, 0));
 #else
 				UNEXPECTED
 #endif
@@ -535,14 +546,16 @@ static int stress_clock(const stress_args_t *args)
 				if ((ret < 0) && (g_opt_flags & OPT_FLAGS_VERIFY) &&
 				    (errno != EINVAL) && (errno != ENOSYS)) {
 					pr_fail("%s: clock_gettime failed for /dev/ptp0, errno=%d (%s)",
-					args->name, errno, strerror(errno));
+						args->name, errno, strerror(errno));
+					rc = EXIT_FAILURE;
 				}
 #if defined(HAVE_CLOCK_GETRES)
 				ret = shim_clock_getres(clkid, &t);
-				if ((ret < 0) && (g_opt_flags & OPT_FLAGS_VERIFY) &&
-				    (errno != EINVAL) && (errno != ENOSYS)) {
+				if (UNLIKELY(((ret < 0) && (g_opt_flags & OPT_FLAGS_VERIFY) &&
+					     (errno != EINVAL) && (errno != ENOSYS)))) {
 					pr_fail("%s: clock_getres failed for /dev/ptp0, errno=%d (%s)",
-					args->name, errno, strerror(errno));
+						args->name, errno, strerror(errno));
+					rc = EXIT_FAILURE;
 				}
 #else
 				UNEXPECTED
@@ -551,24 +564,26 @@ static int stress_clock(const stress_args_t *args)
 			}
 		}
 #endif
-		inc_counter(args);
-	} while (keep_stressing(args));
+		stress_bogo_inc(args);
+	} while (stress_continue(args));
 
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
-	return EXIT_SUCCESS;
+	return rc;
 }
 
-stressor_info_t stress_clock_info = {
+const stressor_info_t stress_clock_info = {
 	.stressor = stress_clock,
-	.class = CLASS_INTERRUPT | CLASS_OS,
+	.classifier = CLASS_INTERRUPT | CLASS_OS,
 	.verify = VERIFY_OPTIONAL,
 	.help = help
 };
 #else
-stressor_info_t stress_clock_info = {
-	.stressor = stress_not_implemented,
-	.class = CLASS_INTERRUPT | CLASS_OS,
-	.help = help
+const stressor_info_t stress_clock_info = {
+	.stressor = stress_unimplemented,
+	.classifier = CLASS_INTERRUPT | CLASS_OS,
+	.verify = VERIFY_OPTIONAL,
+	.help = help,
+	.unimplemented_reason = "built without librt or clock_gettime()/clock_settime() support"
 };
 #endif
