@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C) 2022-2024 Colin Ian King.
+ * Copyright (C) 2022-2025 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -25,14 +25,12 @@
 #endif
 
 static const stress_help_t help[] = {
-	{ NULL,	"crypt N",	"start N workers performing password encryption" },
-	{ NULL,	"crypt-ops N",	"stop after N bogo crypt operations" },
-	{ NULL,	NULL,		NULL }
+	{ NULL,	"crypt N",	  "start N workers performing password encryption" },
+	{ NULL, "crypt-method M", "select encryption method [ all | MD5 | NT | SHA-1 | SHA-256 | SHA-512 | scrypt | SunMD5 | yescrypt]" },
+	{ NULL,	"crypt-ops N",	  "stop after N bogo crypt operations" },
+	{ NULL,	NULL,		  NULL }
 };
 
-#if defined(HAVE_LIB_CRYPT) &&	\
-    defined(HAVE_CRYPT_H)
-
 typedef struct {
 	const char *prefix;
 	const size_t prefix_len;
@@ -40,16 +38,34 @@ typedef struct {
 } crypt_method_t;
 
 static const crypt_method_t crypt_methods[] = {
+	{ NULL,		0,	"all" },
+	{ "$2b$",	4,	"bcrypt" },
+	{ "_",		1,	"bsdicrypt" },
+	{ "",		0,	"descrypt" },
+	{ "$gy$",	4,	"gost-yescrypt" },
 	{ "$1$",	3,	"MD5" },
 	{ "$3$",	3,	"NT" },
+	{ "$7$",	3,	"scrypt" },
 	{ "$sha1",	5,	"SHA-1" },
 	{ "$5$",	3,	"SHA-256" },
 	{ "$6$",	3,	"SHA-512" },
-	{ "$7$",	3,	"scrypt" },
-	{ "$md5",	5,	"SunMD5" },
+	{ "$md5",	4,	"SunMD5" },
 	{ "$y$",	3,	"yescrypt" },
 };
 
+static const char * stress_crypt_method(const size_t i)
+{
+	return (i < SIZEOF_ARRAY(crypt_methods)) ? crypt_methods[i].method : NULL;
+}
+
+static const stress_opt_t opts[] = {
+	{ OPT_crypt_method, "crypt-method", TYPE_ID_SIZE_T_METHOD, 0, 0, stress_crypt_method },
+	END_OPT,
+};
+
+#if defined(HAVE_LIB_CRYPT) &&	\
+    (defined(HAVE_CRYPT_H) || 	\
+     defined(__FreeBSD__))
 static stress_metrics_t *crypt_metrics;
 
 /*
@@ -67,7 +83,7 @@ static int stress_crypt_id(
 #endif
 	)
 {
-	char *encrypted;
+	const char *encrypted;
 	double t1, t2;
 	errno = 0;
 
@@ -116,64 +132,94 @@ static int stress_crypt_id(
  */
 static int stress_crypt(stress_args_t *args)
 {
-	register size_t i;
+	register size_t i, j;
+	size_t crypt_method = 0;	/* all */
+#if defined(HAVE_CRYPT_R)
+	static struct crypt_data data;
+#endif
 
-	crypt_metrics = calloc(SIZEOF_ARRAY(crypt_methods), sizeof(*crypt_metrics));
+	(void)stress_get_setting("crypt-method", &crypt_method);
+
+	crypt_metrics = (stress_metrics_t *)calloc(SIZEOF_ARRAY(crypt_methods), sizeof(*crypt_metrics));
 	if (!crypt_metrics) {
 		pr_inf_skip("%s: cannot allocate crypt metrics "
-			"array, skipping stressor\n",
-			args->name);
+			"array%s, skipping stressor\n",
+			args->name, stress_get_memfree_str());
 		return EXIT_NO_RESOURCE;
 	}
-	for (i = 0; i < SIZEOF_ARRAY(crypt_methods); i++) {
-		crypt_metrics[i].duration = 0.0;
-		crypt_metrics[i].count = 0.0;
-	}
-
+	stress_zero_metrics(crypt_metrics, SIZEOF_ARRAY(crypt_methods));
+#if defined(HAVE_CRYPT_R)
+	(void)shim_memset(&data, 0, sizeof(data));
+#endif
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	do {
-		static const char seedchars[] ALIGN64 =
+		static const char seedchars[64] ALIGN64 NONSTRING =
 			"./0123456789ABCDEFGHIJKLMNOPQRST"
 			"UVWXYZabcdefghijklmnopqrstuvwxyz";
 #if defined(HAVE_CRYPT_R)
-		static struct crypt_data data;
 		char *const phrase = data.input;
 		char *const setting = data.setting;
-		const size_t setting_len = sizeof(data.setting);
 #else
 		char phrase[16];
 		char setting[12];
-		const size_t setting_len = sizeof(setting);
 #endif
 		char orig_setting[12];
-		uint64_t seed[2];
+		char orig_phrase[16];
 		const crypt_method_t *cm = crypt_methods;
 		size_t failed = 0;
+		int ret;
+		size_t n = cm->prefix_len + 8;
 
-		seed[0] = stress_mwc64();
-		seed[1] = stress_mwc64();
+		setting[0] = '$';
+		setting[1] = 'x';
+		setting[2] = '$';
 
-		for (i = 0; i < 8; i++)
-			orig_setting[i + 3] = seedchars[(seed[i / 5] >> (i % 5) * 6) & 0x3f];
-		for (i = 0; i < sizeof(phrase) - 1; i++)
-			phrase[i] = seedchars[stress_mwc32modn((uint32_t)sizeof(seedchars))];
-		phrase[i] = '\0';
+		n = (n > sizeof(orig_setting)) ? sizeof(orig_setting) : n;
 
-		for (i = 0; stress_continue(args) && (i < SIZEOF_ARRAY(crypt_methods)); i++, cm++) {
-			int ret;
+		for (i = 0; i < n; i++)
+			orig_setting[i] = seedchars[stress_mwc8() & 0x3f];
+		orig_setting[n - 1] = '\0';
 
-			(void)shim_strlcpy(setting, orig_setting, setting_len);
+		for (i = 0; i < sizeof(orig_phrase) - 1; i++)
+			orig_phrase[i] = seedchars[stress_mwc8() & 0x3f];
+		orig_phrase[i] = '\0';
+
+		if (crypt_method == 0) {
+			for (i = 1; LIKELY(stress_continue(args) && (i < SIZEOF_ARRAY(crypt_methods))); i++, cm++) {
+				(void)shim_memcpy(setting + cm->prefix_len, orig_setting, n);
+				(void)shim_memcpy(setting, cm->prefix, cm->prefix_len);
+				(void)shim_memcpy(phrase, orig_phrase, sizeof(orig_phrase));
+
+#if defined (HAVE_CRYPT_R)
+				data.initialized = 0;
+#endif
+				ret = stress_crypt_id(args, i,
+#if defined (HAVE_CRYPT_R)
+					&data);
+#else
+					phrase, setting);
+#endif
+				if (UNLIKELY(ret < 0))
+					failed++;
+				else
+					stress_bogo_inc(args);
+			}
+		} else {
+			(void)shim_strscpy(setting, orig_setting, sizeof(orig_setting));
 			(void)shim_memcpy(setting, cm->prefix, cm->prefix_len);
+			(void)shim_memcpy(phrase, orig_phrase, sizeof(orig_phrase));
 #if defined (HAVE_CRYPT_R)
 			data.initialized = 0;
 #endif
 
-			ret = stress_crypt_id(args, i,
+			ret = stress_crypt_id(args, crypt_method,
 #if defined (HAVE_CRYPT_R)
-					      &data);
+				&data);
 #else
-					      phrase, setting);
+				phrase, setting);
 #endif
 			if (UNLIKELY(ret < 0))
 				failed++;
@@ -186,13 +232,17 @@ static int stress_crypt(stress_args_t *args)
 
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
-	for (i = 0; i < SIZEOF_ARRAY(crypt_methods); i++) {
-		char str[40];
+	for (i = 1, j = 0; i < SIZEOF_ARRAY(crypt_methods); i++) {
 		const double duration = crypt_metrics[i].duration;
 		const double rate = duration > 0 ? crypt_metrics[i].count / duration : 0.0;
 
-		(void)snprintf(str, sizeof(str), "%s encrypts per sec", crypt_methods[i].method);
-		stress_metrics_set(args, i, str, rate, STRESS_HARMONIC_MEAN);
+		if (rate > 0.0) {
+			char str[40];
+
+			(void)snprintf(str, sizeof(str), "%s encrypts per sec", crypt_methods[i].method);
+			stress_metrics_set(args, j, str, rate, STRESS_METRIC_HARMONIC_MEAN);
+			j++;
+		}
 	}
 
 	free(crypt_metrics);
@@ -200,18 +250,20 @@ static int stress_crypt(stress_args_t *args)
 	return EXIT_SUCCESS;
 }
 
-stressor_info_t stress_crypt_info = {
+const stressor_info_t stress_crypt_info = {
 	.stressor = stress_crypt,
-	.class = CLASS_CPU,
+	.classifier = CLASS_CPU,
+	.opts = opts,
 	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
-stressor_info_t stress_crypt_info = {
+const stressor_info_t stress_crypt_info = {
 	.stressor = stress_unimplemented,
-	.class = CLASS_CPU,
+	.classifier = CLASS_CPU | CLASS_COMPUTE,
+	.opts = opts,
 	.verify = VERIFY_ALWAYS,
 	.help = help,
-	.unimplemented_reason = "built without gcrypt library"
+	.unimplemented_reason = "built without crypt library"
 };
 #endif
