@@ -22,11 +22,16 @@
 #include "core-capabilities.h"
 #include "core-killpid.h"
 #include "core-lock.h"
+#include "core-mmap.h"
 
 #include <math.h>
 #include <sched.h>
 #include <time.h>
 
+#if defined(HAVE_SYS_SELECT_H)
+#include <sys/select.h>
+#endif
+
 #define DEFAULT_DELAY_NS	(100000)
 #define MAX_SAMPLES		(100000000)
 #define DEFAULT_SAMPLES		(10000)
@@ -326,7 +331,7 @@ static int stress_cyclic_itimer(
 	if (timer_settime(timerid, 0, &timer, NULL) < 0)
 		goto restore;
 
-	(void)pause();
+	(void)shim_pause();
 	if ((itimer_time.tv_sec == 0) &&
 	    (itimer_time.tv_nsec == 0))
 		goto tidy;
@@ -383,6 +388,7 @@ static int stress_cyclic_usleep(
 	UNEXPECTED
 #endif
 
+#if defined(HAVE_SIGLONGJMP)
 static sigjmp_buf jmp_env;
 
 /*
@@ -395,7 +401,9 @@ static void NORETURN MLOCKED_TEXT stress_rlimit_handler(int signum)
 
 	stress_continue_set_flag(false);
 	siglongjmp(jmp_env, 1);
+	stress_no_return();
 }
+#endif
 
 /*
  *  stress_cyclic_cmp()
@@ -586,7 +594,9 @@ static int stress_cyclic_supported(const char *name)
 static int stress_cyclic(stress_args_t *args)
 {
 	const uint32_t instances = args->instances;
+#if defined(HAVE_SIGLONGJMP)
 	struct sigaction old_action_xcpu;
+#endif
 	struct rlimit rlim;
 	pid_t pid;
 	NOCLOBBER uint64_t timeout;
@@ -615,7 +625,7 @@ static int stress_cyclic(stress_args_t *args)
 	if (NUM_CYCLIC_POLICIES == 0) {
 		if (!args->instance) {
 			pr_inf_skip("%s: no scheduling policies "
-				"available, skipping test\n",
+				"available, skipping stressor\n",
 				args->name);
 		}
 		return EXIT_NOT_IMPLEMENTED;
@@ -630,7 +640,7 @@ static int stress_cyclic(stress_args_t *args)
 	if (NUM_CYCLIC_METHODS == 0) {
 		if (!args->instance) {
 			pr_inf_skip("%s: no cyclic methods"
-				"available, skipping test\n",
+				"available, skipping stressor\n",
 				args->name);
 		}
 		return EXIT_NOT_IMPLEMENTED;
@@ -652,7 +662,7 @@ static int stress_cyclic(stress_args_t *args)
 			"be %" PRIu64 " seconds\n", args->name, timeout);
 	}
 
-	if ((instances > 1) && (args->instance == 0)) {
+	if ((instances > 1) && (stress_instance_zero(args))) {
 		pr_inf("%s: for best results, run just 1 instance of "
 			"this stressor\n", args->name);
 	}
@@ -661,8 +671,9 @@ static int stress_cyclic(stress_args_t *args)
 						PROT_READ | PROT_WRITE,
 						MAP_SHARED | MAP_ANONYMOUS, -1, 0);
 	if (rt_stats == MAP_FAILED) {
-		pr_inf_skip("%s: mmap of shared statistics data failed, errno=%d (%s)\n",
-			args->name, errno, strerror(errno));
+		pr_inf_skip("%s: mmap of shared statistics data failed%s, errno=%d (%s)\n",
+			args->name, stress_get_memfree_str(),
+			errno, strerror(errno));
 		return EXIT_NO_RESOURCE;
 	}
 	stress_set_vma_anon_name(rt_stats, size, "rt-statistics");
@@ -673,8 +684,9 @@ static int stress_cyclic(stress_args_t *args)
 						PROT_READ | PROT_WRITE,
 						MAP_SHARED | MAP_ANONYMOUS, -1, 0);
 	if (rt_stats->latencies == MAP_FAILED) {
-		pr_inf_skip("%s: mmap of %zd samples failed, errno=%d (%s)\n",
-			args->name, cyclic_samples, errno, strerror(errno));
+		pr_inf_skip("%s: mmap of %zu samples failed%s, errno=%d (%s)\n",
+			args->name, cyclic_samples, stress_get_memfree_str(),
+			errno, strerror(errno));
 		(void)munmap((void *)rt_stats, size);
 		return EXIT_NO_RESOURCE;
 	}
@@ -700,7 +712,7 @@ static int stress_cyclic(stress_args_t *args)
 		}
 	}
 
-	if (args->instance == 0)
+	if (stress_instance_zero(args))
 		pr_dbg("%s: using method '%s'\n", args->name, cyclic_methods[cyclic_method].name);
 
 	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
@@ -743,12 +755,15 @@ again:
 		(void)setrlimit(RLIMIT_RTTIME, &rlim);
 #endif
 
+#if defined(HAVE_SIGLONGJMP)
 		ret = sigsetjmp(jmp_env, 1);
 		if (ret)
 			goto tidy_ok;
-
 		if (stress_sighandler(args->name, SIGXCPU, stress_rlimit_handler, &old_action_xcpu) < 0)
 			goto tidy;
+#else
+		(void)ret;
+#endif
 
 #if defined(HAVE_SCHED_GET_PRIORITY_MIN) &&	\
     defined(HAVE_SCHED_GET_PRIORITY_MAX)
@@ -817,9 +832,15 @@ redo_policy:
 				break;
 		} while (stress_continue(args));
 
+#if defined(HAVE_SIGLONGJMP)
 tidy_ok:
+#endif
 		ncrc = EXIT_SUCCESS;
+#if defined(HAVE_SIGLONGJMP) ||	\
+    (defined(HAVE_SCHED_GET_PRIORITY_MIN) &&	\
+     defined(HAVE_SCHED_GET_PRIORITY_MAX))
 tidy:
+#endif
 		(void)fflush(stdout);
 		(void)munmap((void *)rt_stats->latencies, rt_stats->latencies_size);
 		(void)munmap((void *)rt_stats, size);
@@ -827,14 +848,14 @@ tidy:
 	} else {
 		VOID_RET(int, stress_set_sched(args->pid, policy, rt_stats->max_prio, true));
 
-		(void)pause();
+		(void)shim_pause();
 		stress_force_killed_bogo(args);
 		(void)stress_kill_pid_wait(pid, NULL);
 	}
 
 	stress_rt_stats(rt_stats);
 
-	if (args->instance == 0) {
+	if (stress_instance_zero(args)) {
 		if (rt_stats->index) {
 			size_t i;
 
@@ -921,7 +942,7 @@ static const stress_opt_t opts[] = {
 const stressor_info_t stress_cyclic_info = {
 	.stressor = stress_cyclic,
 	.supported = stress_cyclic_supported,
-	.class = CLASS_SCHEDULER | CLASS_OS,
+	.classifier = CLASS_SCHEDULER | CLASS_OS,
 	.opts = opts,
 	.init = stress_cyclic_init,
 	.deinit = stress_cyclic_deinit,
