@@ -21,6 +21,7 @@
 #include "core-attribute.h"
 #include "core-builtin.h"
 #include "core-madvise.h"
+#include "core-mmap.h"
 #include "core-put.h"
 #include "core-target-clones.h"
 
@@ -86,7 +87,7 @@ static double stress_dfp_all(
 	dfp_data_t *dfp_data,
 	const int idx);
 
-#define STRESS_DFP_ADD(field, name, do_bogo_ops)				\
+#define STRESS_DFP_ADD(field, name, do_bogo_ops)			\
 static double TARGET_CLONES OPTIMIZE3 name(				\
 	stress_args_t *args,						\
 	dfp_data_t *dfp_data,						\
@@ -126,7 +127,47 @@ static double TARGET_CLONES OPTIMIZE3 name(				\
 	return t2 - t1;							\
 }
 
-#define STRESS_DFP_MUL(field, name, do_bogo_ops)				\
+#define STRESS_DFP_SUB(field, name, do_bogo_ops)			\
+static double TARGET_CLONES OPTIMIZE3 name(				\
+	stress_args_t *args,						\
+	dfp_data_t *dfp_data,						\
+	const int idx)							\
+{									\
+	register int i;							\
+	const int loops = LOOPS_PER_CALL >> 1;				\
+	double t1, t2;							\
+									\
+	for (i = 0; i < DFP_ELEMENTS; i++) {				\
+		dfp_data[i].field.r[idx] = dfp_data[i].field.r_init;	\
+	}								\
+									\
+	t1 = stress_time_now();						\
+	for (i = 0; i < loops ; i++) {					\
+		dfp_data[0].field.r[idx] -= dfp_data[0].field.add;	\
+		dfp_data[0].field.r[idx] -= dfp_data[0].field.add_rev;	\
+		dfp_data[1].field.r[idx] -= dfp_data[1].field.add;	\
+		dfp_data[1].field.r[idx] -= dfp_data[1].field.add_rev;	\
+		dfp_data[2].field.r[idx] -= dfp_data[2].field.add;	\
+		dfp_data[2].field.r[idx] -= dfp_data[2].field.add_rev;	\
+		dfp_data[3].field.r[idx] -= dfp_data[3].field.add;	\
+		dfp_data[3].field.r[idx] -= dfp_data[3].field.add_rev;	\
+		dfp_data[4].field.r[idx] -= dfp_data[4].field.add;	\
+		dfp_data[4].field.r[idx] -= dfp_data[4].field.add_rev;	\
+		dfp_data[5].field.r[idx] -= dfp_data[5].field.add;	\
+		dfp_data[5].field.r[idx] -= dfp_data[5].field.add_rev;	\
+		dfp_data[6].field.r[idx] -= dfp_data[6].field.add;	\
+		dfp_data[6].field.r[idx] -= dfp_data[6].field.add_rev;	\
+		dfp_data[7].field.r[idx] -= dfp_data[7].field.add;	\
+		dfp_data[7].field.r[idx] -= dfp_data[7].field.add_rev;	\
+	}								\
+	t2 = stress_time_now();						\
+									\
+	if (do_bogo_ops)						\
+		stress_bogo_inc(args);					\
+	return t2 - t1;							\
+}
+
+#define STRESS_DFP_MUL(field, name, do_bogo_ops)			\
 static double TARGET_CLONES OPTIMIZE3 name(				\
 	stress_args_t *args,						\
 	dfp_data_t *dfp_data,						\
@@ -166,7 +207,7 @@ static double TARGET_CLONES OPTIMIZE3 name(				\
 	return t2 - t1;							\
 }
 
-#define STRESS_DFP_DIV(field, name, do_bogo_ops)				\
+#define STRESS_DFP_DIV(field, name, do_bogo_ops)			\
 static double TARGET_CLONES OPTIMIZE3 name(				\
 	stress_args_t *args,						\
 	dfp_data_t *dfp_data,						\
@@ -208,18 +249,21 @@ static double TARGET_CLONES OPTIMIZE3 name(				\
 
 #if defined(HAVE_Decimal32)
 STRESS_DFP_ADD(d32, stress_dfp_d32_add, true)
+STRESS_DFP_ADD(d32, stress_dfp_d32_sub, true)
 STRESS_DFP_MUL(d32, stress_dfp_d32_mul, true)
 STRESS_DFP_DIV(d32, stress_dfp_d32_div, true)
 #endif
 
 #if defined(HAVE_Decimal64)
 STRESS_DFP_ADD(d64, stress_dfp_d64_add, true)
+STRESS_DFP_ADD(d64, stress_dfp_d64_sub, true)
 STRESS_DFP_MUL(d64, stress_dfp_d64_mul, true)
 STRESS_DFP_DIV(d64, stress_dfp_d64_div, true)
 #endif
 
 #if defined(HAVE_Decimal128)
 STRESS_DFP_ADD(d128, stress_dfp_d128_add, true)
+STRESS_DFP_ADD(d128, stress_dfp_d128_sub, true)
 STRESS_DFP_MUL(d128, stress_dfp_d128_mul, true)
 STRESS_DFP_DIV(d128, stress_dfp_d128_div, true)
 #endif
@@ -229,43 +273,54 @@ typedef struct {
 	const char *description;
 	const stress_dfp_func_t	dfp_func;
 	const int dfp_type;
-	double duration;
-	double ops;
 } stress_dfp_funcs_t;
 
-static stress_dfp_funcs_t stress_dfp_funcs[] = {
-	{ "all",		"all fp methods",	stress_dfp_all,		STRESS_DFP_TYPE_ALL,		0.0, 0.0 },
+static const stress_dfp_funcs_t stress_dfp_funcs[] = {
+	{ "all",	"all fp methods",		stress_dfp_all,		STRESS_DFP_TYPE_ALL },
 #if defined(HAVE_Decimal32)
-	{ "df32add",	"_Decimal32 add",		stress_dfp_d32_add,	STRESS_DFP_TYPE_DECIMAL32,	0.0, 0.0 },
+	{ "df32add",	"_Decimal32 addition",		stress_dfp_d32_add,	STRESS_DFP_TYPE_DECIMAL32 },
 #endif
 #if defined(HAVE_Decimal64)
-	{ "df64add",	"_Decimal64 add",		stress_dfp_d64_add,	STRESS_DFP_TYPE_DECIMAL64,	0.0, 0.0 },
+	{ "df64add",	"_Decimal64 addition",		stress_dfp_d64_add,	STRESS_DFP_TYPE_DECIMAL64 },
 #endif
 #if defined(HAVE_Decimal128)
-	{ "df128add",	"_Decimal128 add",		stress_dfp_d128_add,	STRESS_DFP_TYPE_DECIMAL128,	0.0, 0.0 },
+	{ "df128add",	"_Decimal128 addition",		stress_dfp_d128_add,	STRESS_DFP_TYPE_DECIMAL128 },
 #endif
+
 #if defined(HAVE_Decimal32)
-	{ "df32mul",	"_Decimal32 mul",		stress_dfp_d32_mul,	STRESS_DFP_TYPE_DECIMAL32,	0.0, 0.0 },
+	{ "df32sub",	"_Decimal32 subtraction",	stress_dfp_d32_sub,	STRESS_DFP_TYPE_DECIMAL32 },
 #endif
 #if defined(HAVE_Decimal64)
-	{ "df64mul",	"_Decimal64 mul",		stress_dfp_d64_mul,	STRESS_DFP_TYPE_DECIMAL64,	0.0, 0.0 },
+	{ "df64sub",	"_Decimal64 subtraction",	stress_dfp_d64_sub,	STRESS_DFP_TYPE_DECIMAL64 },
 #endif
 #if defined(HAVE_Decimal128)
-	{ "df128mul",	"_Decimal128 mul",		stress_dfp_d128_mul,	STRESS_DFP_TYPE_DECIMAL128,	0.0, 0.0 },
+	{ "df128sub",	"_Decimal128 subtraction",	stress_dfp_d128_sub,	STRESS_DFP_TYPE_DECIMAL128 },
 #endif
+
 #if defined(HAVE_Decimal32)
-	{ "df32div",	"_Decimal32 div",		stress_dfp_d32_div,	STRESS_DFP_TYPE_DECIMAL32,	0.0, 0.0 },
+	{ "df32mul",	"_Decimal32 multiplication",	stress_dfp_d32_mul,	STRESS_DFP_TYPE_DECIMAL32 },
 #endif
 #if defined(HAVE_Decimal64)
-	{ "df64div",	"_Decimal64 div",		stress_dfp_d64_div,	STRESS_DFP_TYPE_DECIMAL64,	0.0, 0.0 },
+	{ "df64mul",	"_Decimal64 multiplication",	stress_dfp_d64_mul,	STRESS_DFP_TYPE_DECIMAL64 },
 #endif
 #if defined(HAVE_Decimal128)
-	{ "df128div",	"_Decimal128 div",		stress_dfp_d128_div,	STRESS_DFP_TYPE_DECIMAL128,	0.0, 0.0 },
+	{ "df128mul",	"_Decimal128 multiplication",	stress_dfp_d128_mul,	STRESS_DFP_TYPE_DECIMAL128 },
+#endif
+#if defined(HAVE_Decimal32)
+	{ "df32div",	"_Decimal32 division",		stress_dfp_d32_div,	STRESS_DFP_TYPE_DECIMAL32 },
+#endif
+#if defined(HAVE_Decimal64)
+	{ "df64div",	"_Decimal64 division",		stress_dfp_d64_div,	STRESS_DFP_TYPE_DECIMAL64 },
+#endif
+#if defined(HAVE_Decimal128)
+	{ "df128div",	"_Decimal128 division",		stress_dfp_d128_div,	STRESS_DFP_TYPE_DECIMAL128 },
 #endif
 };
 
 #define STRESS_NUM_DFP_FUNCS	(SIZEOF_ARRAY(stress_dfp_funcs))
 
+static stress_metrics_t stress_dfp_metrics[SIZEOF_ARRAY(stress_dfp_funcs)];
+
 typedef struct {
 	const int dfp_type;
 	const char *dfp_description;
@@ -296,11 +351,12 @@ static int stress_dfp_call_method(
 	const bool verify)
 {
 	double dt;
-	stress_dfp_funcs_t *func = &stress_dfp_funcs[method];
+	stress_dfp_funcs_t const *func = &stress_dfp_funcs[method];
+	stress_metrics_t *metrics = &stress_dfp_metrics[method];
 
 	dt = func->dfp_func(args, dfp_data, 0);
-	func->duration += dt;
-	func->ops += (DFP_ELEMENTS * LOOPS_PER_CALL);
+	metrics->duration += dt;
+	metrics->count += (DFP_ELEMENTS * LOOPS_PER_CALL);
 
 	if ((method > 0) && (method < STRESS_NUM_DFP_FUNCS && verify)) {
 		register size_t i;
@@ -311,8 +367,8 @@ static int stress_dfp_call_method(
 		dt = func->dfp_func(args, dfp_data, 1);
 		if (UNLIKELY(dt < 0.0))
 			return EXIT_FAILURE;
-		func->duration += dt;
-		func->ops += (DFP_ELEMENTS * LOOPS_PER_CALL);
+		metrics->duration += dt;
+		metrics->count += (DFP_ELEMENTS * LOOPS_PER_CALL);
 
 		/*
 		 *  a SIGALRM during 2nd computation pre-verification can
@@ -379,8 +435,8 @@ static int stress_dfp(stress_args_t *args)
 			PROT_READ | PROT_WRITE,
 			MAP_PRIVATE | MAP_ANONYMOUS, -1, 0);
 	if (dfp_data == MAP_FAILED) {
-		pr_inf_skip("%s: failed to allocate %d decimal floating point elements, skipping stressor\n",
-			args->name, DFP_ELEMENTS);
+		pr_inf_skip("%s: failed to allocate %d decimal floating point elements%s, skipping stressor\n",
+			args->name, DFP_ELEMENTS, stress_get_memfree_str());
 		return EXIT_NO_RESOURCE;
 	}
 	stress_set_vma_anon_name(dfp_data, mmap_size, "dfp-data");
@@ -392,6 +448,11 @@ static int stress_dfp(stress_args_t *args)
 	stress_sync_start_wait(args);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
+	for (i = 0; i < SIZEOF_ARRAY(stress_dfp_metrics); i++) {
+		stress_dfp_metrics[i].duration = 0.0;
+		stress_dfp_metrics[i].count = 0.0;
+	}
+
 	for (i = 0; i < DFP_ELEMENTS; i++) {
 		uint32_t r;
 #if defined(HAVE_Decimal32)
@@ -487,11 +548,11 @@ static int stress_dfp(stress_args_t *args)
 	} while (stress_continue(args));
 
 	for (i = 1; i < STRESS_NUM_DFP_FUNCS; i++) {
-		const double ops = stress_dfp_funcs[i].ops;
-		const double duration = stress_dfp_funcs[i].duration;
-		if ((duration > 0.0) && (ops > 0.0)) {
+		const double count = stress_dfp_metrics[i].count;
+		const double duration = stress_dfp_metrics[i].duration;
+		if ((duration > 0.0) && (count > 0.0)) {
 			char msg[64];
-			const double rate = stress_dfp_funcs[i].ops / stress_dfp_funcs[i].duration;
+			const double rate = count / duration;
 
 			(void)snprintf(msg, sizeof(msg), "Mdfp-ops per sec, %-20s", stress_dfp_funcs[i].description);
 			stress_metrics_set(args, i - 1, msg,
@@ -518,7 +579,7 @@ static const stress_opt_t opts[] = {
 
 const stressor_info_t stress_dfp_info = {
 	.stressor = stress_dfp,
-	.class = CLASS_CPU | CLASS_FP | CLASS_COMPUTE,
+	.classifier = CLASS_CPU | CLASS_FP | CLASS_COMPUTE,
 	.opts = opts,
 	.verify = VERIFY_OPTIONAL,
 	.help = help
@@ -539,7 +600,7 @@ static const stress_opt_t opts[] = {
 
 const stressor_info_t stress_dfp_info = {
 	.stressor = stress_unimplemented,
-	.class = CLASS_CPU | CLASS_FP | CLASS_COMPUTE,
+	.classifier = CLASS_CPU | CLASS_FP | CLASS_COMPUTE,
 	.opts = opts,
 	.verify = VERIFY_OPTIONAL,
 	.help = help,
