@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C)      2022 Colin Ian King.
+ * Copyright (C) 2022-2025 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -18,6 +18,12 @@
  *
  */
 #include "stress-ng.h"
+#include "core-killpid.h"
+#include "core-out-of-memory.h"
+
+#if defined(HAVE_SYS_PARAM_H)
+#include <sys/param.h>
+#endif
 
 static const stress_help_t help[] = {
 	{ NULL,	"env N",	"start N workers setting environment vars" },
@@ -27,7 +33,7 @@ static const stress_help_t help[] = {
 
 static size_t stress_env_size(const size_t arg_max)
 {
-	return (size_t)(1 + (stress_mwc32() % (arg_max - 2)));
+	return (size_t)(1 + stress_mwc32modn((uint32_t)(arg_max - 2)));
 }
 
 static uint64_t stress_env_max(void)
@@ -35,7 +41,7 @@ static uint64_t stress_env_max(void)
 	return stress_mwc1() ? ~(uint64_t)0 : (uint64_t)(stress_mwc16());
 }
 
-static int stress_env_child(const stress_args_t *args, void *context)
+static int stress_env_child(stress_args_t *args, void *context)
 {
 	const size_t page_size = args->page_size;
 	uint64_t i = 0;
@@ -45,6 +51,7 @@ static int stress_env_child(const stress_args_t *args, void *context)
 	const size_t arg_huge = 16 * MB;
 	char *value;
 	const bool verify = !!(g_opt_flags & OPT_FLAGS_VERIFY);
+	int rc = EXIT_SUCCESS;
 
 	(void)context;
 
@@ -74,18 +81,21 @@ static int stress_env_child(const stress_args_t *args, void *context)
 		value = (char *)mmap(NULL, arg_max, PROT_READ | PROT_WRITE,
 			MAP_PRIVATE | MAP_ANONYMOUS, -1, 0);
 		if (value == MAP_FAILED) {
-			pr_inf_skip("%s: could not allocate %zd bytes for environment variable value, "
-				"errno = %d (%s), skipping stressor\n",
-				args->name, arg_max,
+			pr_inf_skip("%s: could not allocate %zu bytes for environment variable value%s, "
+				"errno=%d (%s), skipping stressor\n",
+				args->name, arg_max, stress_get_memfree_str(),
 				errno, strerror(errno));
 			return EXIT_NO_RESOURCE;
 		}
-		pr_inf("%s: falling back to %zd byte sized environment variable value size\n",
+		pr_inf("%s: falling back to %zu byte sized environment variable value size\n",
 			args->name, arg_max);
 	}
+	stress_set_vma_anon_name(value, arg_max, "env-variable-value");
 
 	stress_mwc_reseed();
-	stress_strnrnd(value, arg_max);
+	stress_rndstr(value, arg_max);
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	env_max = stress_env_max();
@@ -108,7 +118,7 @@ static int stress_env_child(const stress_args_t *args, void *context)
 
 		/* Low memory avoidance, re-start */
 		if (stress_low_memory(arg_max * 2)) {
-			(void)kill(getpid(), SIGKILL);
+			(void)stress_kill_pid(getpid());
 			_exit(EXIT_SUCCESS);
 		}
 
@@ -120,19 +130,21 @@ static int stress_env_child(const stress_args_t *args, void *context)
 			for (j = 0; j < i; j++) {
 				if (verify) {
 					const size_t env_sz = stress_env_size(arg_max);
-					char *val;
+					const char *val;
 
 					(void)snprintf(name, sizeof(name), "STRESS_ENV_%" PRIx64, j);
 					val = getenv(name);
 					if (!val) {
 						pr_fail("%s: cannot fetch environment variable %s\n",
 							args->name, name);
+						rc = EXIT_FAILURE;
 					} else {
 						tmp = value[env_sz];
 						value[env_sz] = '\0';
 						if (strcmp(value, val)) {
 							pr_fail("%s: environment variable %s contains incorrect data\n",
 								args->name, name);
+							rc = EXIT_FAILURE;
 						}
 						value[env_sz] = tmp;
 					}
@@ -141,9 +153,10 @@ static int stress_env_child(const stress_args_t *args, void *context)
 				if (ret < 0) {
 					pr_fail("%s: unsentenv on variable %s failed, errno=%d (%s)\n",
 						args->name, name, errno, strerror(errno));
+					rc = EXIT_FAILURE;
 				}
-				inc_counter(args);
-				if (!keep_stressing(args))
+				stress_bogo_inc(args);
+				if (UNLIKELY(!stress_continue(args)))
 					goto reap;
 			}
 			i = 0;
@@ -151,30 +164,30 @@ static int stress_env_child(const stress_args_t *args, void *context)
 			stress_mwc_get_seed(&seed_w, &seed_z);
 		} else {
 			i++;
-			inc_counter(args);
+			stress_bogo_inc(args);
 		}
-	} while (keep_stressing(args));
+	} while (stress_continue(args));
 
 reap:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
-	(void)munmap(value, arg_max);
+	(void)munmap((void *)value, arg_max);
 
-	return EXIT_SUCCESS;
+	return rc;
 }
 
 /*
  *  stress_env()
  *	stress environment variables
  */
-static int stress_env(const stress_args_t *args)
+static int stress_env(stress_args_t *args)
 {
 	return stress_oomable_child(args, NULL, stress_env_child, STRESS_OOMABLE_DROP_CAP | STRESS_OOMABLE_QUIET);
 }
 
-stressor_info_t stress_env_info = {
+const stressor_info_t stress_env_info = {
 	.stressor = stress_env,
-	.class = CLASS_OS | CLASS_VM,
+	.classifier = CLASS_OS | CLASS_VM,
 	.verify = VERIFY_OPTIONAL,
 	.help = help
 };
