@@ -22,10 +22,13 @@
 #include "core-out-of-memory.h"
 #include "core-pthread.h"
 
-#include <sys/file.h>
 #include <sys/ioctl.h>
 #include <ctype.h>
 
+#if defined(HAVE_SYS_FILE_H)
+#include <sys/file.h>
+#endif
+
 #if defined(HAVE_SYS_SELECT_H)
 #include <sys/select.h>
 #endif
@@ -297,6 +300,12 @@ retry:
 			return EXIT_NO_RESOURCE;
 
 		if (UNLIKELY((fd = socket(AF_UNIX, SOCK_STREAM, 0)) < 0)) {
+			if ((errno == ENFILE) ||
+			    (errno == ENOBUFS) ||
+			    (errno == ENOMEM)) {
+				stress_random_small_sleep();
+				goto retry;
+			}
 			pr_fail("%s: socket failed, errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
 			return EXIT_FAILURE;
@@ -363,7 +372,6 @@ retry:
 	UNEXPECTED
 #endif
 	return EXIT_SUCCESS;
-
 }
 
 /*
@@ -501,7 +509,17 @@ static int OPTIMIZE3 stress_race_fd_server(
 		rc = EXIT_FAILURE;
 		goto die;
 	}
+
+retry:
+	if (UNLIKELY(!stress_continue_flag()))
+		goto die;
 	if ((fd = socket(AF_UNIX, SOCK_STREAM, 0)) < 0) {
+		if ((errno == ENFILE) ||
+		    (errno == ENOBUFS) ||
+		    (errno == ENOMEM)) {
+			stress_random_small_sleep();
+			goto retry;
+		}
 		rc = stress_exit_status(errno);
 		pr_fail("%s: socket failed, errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
@@ -514,7 +532,6 @@ static int OPTIMIZE3 stress_race_fd_server(
 		rc = EXIT_FAILURE;
 		goto die_close;
 	}
-
 	if (stress_set_sockaddr(args->name, args->instance, context->pid,
 			AF_UNIX, context->socket_fd_port,
 			&addr, &addr_len, NET_ADDR_ANY) < 0) {
@@ -646,7 +663,7 @@ static stress_fd_race_filename_t *stress_fd_race_filename_add(
 	if (!entry)
 		return NULL;
 
-	entry->filename = strdup(filename);
+	entry->filename = shim_strdup(filename);
 	if (!entry->filename) {
 		free(entry);
 		return NULL;
@@ -699,6 +716,7 @@ static void stress_fd_race_filename_dir(const char *dirname, stress_fd_race_file
 	while ((de = readdir(dir)) != NULL) {
 		struct stat statbuf;
 		ssize_t len;
+		int val;
 
 		/* ignore special dirs */
 		if ((de->d_name[0] == '\0') || (de->d_name[0] == '.'))
@@ -708,10 +726,10 @@ static void stress_fd_race_filename_dir(const char *dirname, stress_fd_race_file
 			if (!isdigit((unsigned char)de->d_name[len]))
 				break;
 		}
-		/* ignore numbered files such as /dev/tty1 upwards */
-		if (atoi(&de->d_name[len + 1]) > 0)
-			continue;
-
+		/* allow /dev/tty, /dev/tty0 ignore numbered files such as /dev/tty1 upwards */
+		if (sscanf(&de->d_name[len + 1], "%d", &val) == 1)
+			if (val > 0)
+				continue;
 		if (fstatat(dir_fd, de->d_name, &statbuf, 0) < 0)
 			continue;
 
@@ -848,8 +866,8 @@ static int stress_fd_race(stress_args_t *args)
 	context.fds_size = sizeof(*context.fds) * (size_t)context.max_fd;
 	context.fds = (int *)malloc(context.fds_size);
 	if (!context.fds) {
-		pr_inf_skip("%s: cannot allocate %zd file descriptors, skipping stressor\n",
-			args->name, context.max_fd);
+		pr_inf_skip("%s: cannot allocate %zd file descriptors%s, skipping stressor\n",
+			args->name, context.max_fd, stress_get_memfree_str());
 		rc = EXIT_NO_RESOURCE;
 		goto tidy_file;
 	}
@@ -902,7 +920,7 @@ tidy_dir:
 
 const stressor_info_t stress_fd_race_info = {
 	.stressor = stress_fd_race,
-	.class = CLASS_OS,
+	.classifier = CLASS_OS,
 	.verify = VERIFY_ALWAYS,
 	.opts = opts,
 	.help = help
@@ -910,7 +928,7 @@ const stressor_info_t stress_fd_race_info = {
 #else
 const stressor_info_t stress_fd_race_info = {
 	.stressor = stress_unimplemented,
-	.class = CLASS_OS,
+	.classifier = CLASS_OS,
 	.verify = VERIFY_ALWAYS,
 	.help = help,
 	.opts = opts,
