@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C) 2022-2023 Colin Ian King.
+ * Copyright (C) 2022-2025 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -18,7 +18,16 @@
  *
  */
 #include "stress-ng.h"
+#include "core-attribute.h"
+#include "core-builtin.h"
+#include "core-helper.h"
+#include "core-madvise.h"
+#include "core-mmap.h"
 #include "core-put.h"
+#include "core-target-clones.h"
+#include "core-pragma.h"
+
+#include <sys/ioctl.h>
 
 #if defined(HAVE_LINUX_FS_H)
 #include <linux/fs.h>
@@ -30,8 +39,12 @@ static const stress_help_t help[] = {
 	{ NULL,	NULL,		NULL }
 };
 
-#if defined(__linux__)
+#if defined(__linux__)	||	\
+    defined(__sun__) ||		\
+    defined(__FreeBSD__) ||	\
+    defined(__NetBSD__)
 
+#if defined(__linux__)
 typedef struct {
 	const char *name;
 	const int whence;
@@ -42,102 +55,115 @@ static const stress_whences_t whences[] = {
 	{ "SEEK_CUR",	SEEK_CUR },
 	{ "SEEK_END",	SEEK_END }
 };
+#endif
 
 /*
  *  stress_full
  *	stress /dev/full
  */
-static int stress_full(const stress_args_t *args)
+static int stress_full(stress_args_t *args)
 {
+	void *buffer;
+	const size_t buffer_size = 4096;
+#if defined(__linux__)
+	size_t w = 0;
+#endif
+	int rc = EXIT_FAILURE;
+	int fd = -1;
+
+	buffer = stress_mmap_populate(NULL, buffer_size,
+			PROT_READ | PROT_WRITE,
+			MAP_ANONYMOUS | MAP_PRIVATE, -1, 0);
+	if (buffer == MAP_FAILED) {
+		pr_inf_skip("%s: failed to mmap %zu bytes%s, errno=%d (%s), skipping stressor\n",
+			args->name, buffer_size, stress_get_memfree_str(),
+			errno, strerror(errno));
+		return EXIT_NO_RESOURCE;
+	}
+	stress_set_vma_anon_name(buffer, buffer_size, "io-buffer");
+	(void)stress_madvise_mergeable(buffer, buffer_size);
+
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	do {
 		ssize_t ret;
-		int fd, w;
-		ssize_t i;
-		off_t offset;
-		char ALIGN64 buffer[4096];
 		uint8_t *ptr;
 		struct stat statbuf;
 
 		if ((fd = open("/dev/full", O_RDWR)) < 0) {
 			if (errno == ENOENT) {
-				if (args->instance == 0)
-					pr_inf_skip("%s: /dev/full not available, skipping stress test\n",
+				if (stress_instance_zero(args))
+					pr_inf_skip("%s: /dev/full not available, skipping stressor\n",
 						args->name);
-				return EXIT_NOT_IMPLEMENTED;
+				rc = EXIT_NOT_IMPLEMENTED;
+				goto fail;
 			}
 			pr_fail("%s: open /dev/full failed, errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
-			return EXIT_FAILURE;
+			goto fail;
 		}
 
 		/*
 		 *  Writes should always return -ENOSPC
 		 */
-		(void)memset(buffer, 0, sizeof(buffer));
-		ret = write(fd, buffer, sizeof(buffer));
-		if (ret != -1) {
+		ret = write(fd, buffer, buffer_size);
+		if (UNLIKELY(ret != -1)) {
 			pr_fail("%s: write to /dev/null should fail "
 				"with errno ENOSPC but it didn't\n",
 				args->name);
-			(void)close(fd);
-			return EXIT_FAILURE;
+			goto fail;
 		}
 		if ((errno == EAGAIN) || (errno == EINTR))
 			goto try_read;
 		if (errno != ENOSPC) {
 			pr_fail("%s: write failed, errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
-			(void)close(fd);
-			return EXIT_FAILURE;
+			goto fail;
 		}
 
 try_read:
 		/*
 		 *  Reads should always work
 		 */
-		ret = read(fd, buffer, sizeof(buffer));
-		if (ret < 0) {
+		ret = read(fd, buffer, buffer_size);
+		if (UNLIKELY(ret < 0)) {
 			pr_fail("%s: read failed, errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
-			(void)close(fd);
-			return EXIT_FAILURE;
+			goto fail;
 		}
-		for (i = 0; i < ret; i++) {
-			if (buffer[i] != 0) {
-				pr_fail("%s: buffer does not contain all zeros\n",
-					args->name);
-				(void)close(fd);
-				return EXIT_FAILURE;
-			}
+		if (stress_data_is_not_zero(buffer, buffer_size)) {
+			pr_fail("%s: buffer does not contain all zeros\n", args->name);
+			goto fail;
 		}
 #if defined(HAVE_PREAD)
 		{
-			offset = (sizeof(offset) == sizeof(uint64_t)) ?
+			const off_t offset = (sizeof(offset) == sizeof(uint64_t)) ?
 				(off_t)(stress_mwc64() & 0x7fffffffffffffff) :
 				(off_t)(stress_mwc32() & 0x7fffffffUL);
-			ret = pread(fd, buffer, sizeof(buffer), offset);
-			if (ret < 0) {
-				pr_fail("%s: read failed at offset %jd, errno=%d (%s)\n",
+			ret = pread(fd, buffer, buffer_size, offset);
+			if (UNLIKELY(ret < 0)) {
+				pr_fail("%s: read failed at offset %" PRIdMAX ", errno=%d (%s)\n",
 					args->name, (intmax_t)offset, errno, strerror(errno));
-				(void)close(fd);
-				return EXIT_FAILURE;
+				goto fail;
 			}
 		}
 #endif
 		/*
 		 *  Try fstat
 		 */
-		ret = fstat(fd, &statbuf);
-		if (ret < 0)
+		ret = shim_fstat(fd, &statbuf);
+		if (UNLIKELY(ret < 0)) {
 			pr_fail("%s: fstat failed, errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
+			goto fail;
+		}
 		/*
 		 *  Try mmap'ing and msync on fd
 		 */
 		ptr = (uint8_t *)mmap(NULL, args->page_size, PROT_READ,
-			MAP_ANONYMOUS | MAP_PRIVATE, fd, 0);
+			MAP_PRIVATE, fd, 0);
 		if (ptr != MAP_FAILED) {
 			stress_uint8_put(*ptr);
 #if defined(MS_SYNC)
@@ -148,24 +174,31 @@ try_read:
 			(void)munmap((void *)ptr, args->page_size);
 		}
 		ptr = (uint8_t *)mmap(NULL, args->page_size, PROT_WRITE,
-			MAP_ANONYMOUS | MAP_PRIVATE, fd, 0);
+			MAP_PRIVATE, fd, 0);
 		if (ptr != MAP_FAILED) {
 			*ptr = 0;
 			(void)munmap((void *)ptr, args->page_size);
 		}
 
-		/*
-		 *  Seeks will always succeed
-		 */
-		w = stress_mwc32modn(3);
-		offset = (off_t)stress_mwc64();
-		ret = lseek(fd, offset, whences[w].whence);
-		if (ret < 0) {
-			pr_fail("%s: lseek(fd, %jd, %s)\n",
-				args->name, (intmax_t)offset, whences[w].name);
-			(void)close(fd);
-			return EXIT_FAILURE;
+#if defined(__linux__)
+		{
+			/*
+			 *  On Linux seeks will always succeed
+			 */
+			const off_t offset = (off_t)stress_mwc64();
+
+			ret = lseek(fd, offset, whences[w].whence);
+			if (ret < 0) {
+				pr_fail("%s: lseek(fd, %" PRIdMAX ", %s) failed, errno=%d (%s)\n",
+					args->name, (intmax_t)offset, whences[w].name,
+					errno, strerror(errno));
+				goto fail;
+			}
 		}
+		w++;
+		if (w >= SIZEOF_ARRAY(whences))
+			w = 0;
+#endif
 
 		/*
 		 *  Exercise a couple of ioctls
@@ -186,24 +219,33 @@ try_read:
 		}
 #endif
 		(void)close(fd);
-		inc_counter(args);
-	} while (keep_stressing(args));
+		fd = -1;
+		stress_bogo_inc(args);
+	} while (stress_continue(args));
 
+	rc = EXIT_SUCCESS;
+
+fail:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
-	return EXIT_SUCCESS;
+	if (fd >= 0)
+		(void)close(fd);
+	(void)munmap((void *)buffer, buffer_size);
+
+	return rc;
 }
 
-stressor_info_t stress_full_info = {
+const stressor_info_t stress_full_info = {
 	.stressor = stress_full,
-	.class = CLASS_DEV | CLASS_MEMORY | CLASS_OS,
+	.classifier = CLASS_DEV | CLASS_MEMORY | CLASS_OS,
 	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
-stressor_info_t stress_full_info = {
+const stressor_info_t stress_full_info = {
 	.stressor = stress_unimplemented,
-	.class = CLASS_DEV | CLASS_MEMORY | CLASS_OS,
+	.classifier = CLASS_DEV | CLASS_MEMORY | CLASS_OS,
+	.verify = VERIFY_ALWAYS,
 	.help = help,
 	.unimplemented_reason = "only supported on Linux"
 };
