@@ -21,6 +21,7 @@
 #include "core-attribute.h"
 #include "core-cpu-cache.h"
 #include "core-madvise.h"
+#include "core-mmap.h"
 #include "core-pragma.h"
 
 #define DEBUG_TAG_INFO		(0)
@@ -42,7 +43,7 @@ static const stress_help_t help[] = {
  *  stress_l1cache_ln2()
  *	calculate log base 2
  */
-static uint32_t PURE stress_l1cache_ln2(const uint32_t val)
+static uint32_t CONST stress_l1cache_ln2(const uint32_t val)
 {
 	int ln2 = 0;
 
