@@ -59,6 +59,42 @@ static const stress_help_t help[] = {
 	{ NULL,	NULL,		     NULL }
 };
 
+static void stress_mmap_stressful(const char *opt_name, const char *opt_arg, stress_type_id_t *type_id, void *value)
+{
+	(void)opt_name;
+	*type_id = TYPE_ID_SIZE_T;
+	*(size_t *)value = 0;
+
+	(void)stress_set_setting_true("mmap", "mmap-mergeable", opt_arg);
+	(void)stress_set_setting_true("mmap", "mmap-mprotect", opt_arg);
+	(void)stress_set_setting_true("mmap", "mmap-file", opt_arg);
+	(void)stress_set_setting_true("mmap", "mmap-odirect", opt_arg);
+	(void)stress_set_setting_true("mmap", "mmap-madvise", opt_arg);
+	(void)stress_set_setting_true("mmap", "mmap-mlock", opt_arg);
+	(void)stress_set_setting_true("mmap", "mmap-numa", opt_arg);
+	(void)stress_set_setting_true("mmap", "mmap-slow-munmap", opt_arg);
+}
+
+static const stress_opt_t opts[] = {
+	{ OPT_mmap_async,       "mmap-async",       TYPE_ID_BOOL, 0, 1, NULL },
+	{ OPT_mmap_bytes,       "mmap-bytes",       TYPE_ID_SIZE_T_BYTES_VM, MIN_MMAP_BYTES, MAX_MMAP_BYTES, NULL },
+	{ OPT_mmap_file,        "mmap-file",        TYPE_ID_BOOL, 0, 1, NULL },
+	{ OPT_mmap_madvise,     "mmap-madvise",     TYPE_ID_BOOL, 0, 1, NULL },
+	{ OPT_mmap_mergeable,   "mmap-mergeable",   TYPE_ID_BOOL, 0, 1, NULL },
+	{ OPT_mmap_mlock,       "mmap-mlock",       TYPE_ID_BOOL, 0, 1, NULL },
+	{ OPT_mmap_mmap2,       "mmap-mmap2",       TYPE_ID_BOOL, 0, 1, NULL },
+	{ OPT_mmap_mprotect,    "mmap-mprotect",    TYPE_ID_BOOL, 0, 1, NULL },
+	{ OPT_mmap_numa,	"mmap-numa",	    TYPE_ID_BOOL, 0, 1, NULL },
+	{ OPT_mmap_odirect,     "mmap-odirect",     TYPE_ID_BOOL, 0, 1, NULL },
+	{ OPT_mmap_osync,       "mmap-osync",       TYPE_ID_BOOL, 0, 1, NULL },
+	{ OPT_mmap_slow_munmap,	"mmap-slow-munmap", TYPE_ID_BOOL, 0, 1, NULL },
+	{ OPT_mmap_stressful,   "mmap-stressful",   TYPE_ID_CALLBACK, 0, 0, stress_mmap_stressful },
+	{ OPT_mmap_write_check, "mmap-write-check", TYPE_ID_BOOL, 0, 1, NULL },
+	END_OPT,
+};
+
+#if defined(HAVE_SIGLONGJMP)
+
 typedef void * (*mmap_func_t)(void *addr, size_t length, int prot, int flags, int fd, off_t offset);
 
 typedef struct {
@@ -82,6 +118,7 @@ typedef struct {
 	int *mmap_flag_perms;
 #if defined(HAVE_LINUX_MEMPOLICY_H)
 	stress_numa_mask_t *numa_mask;
+	stress_numa_mask_t *numa_nodes;
 #endif
 } stress_mmap_context_t;
 
@@ -159,7 +196,8 @@ static const int mmap_flags[] = {
 #if defined(MAP_NORESERVE)
 	MAP_NORESERVE,
 #endif
-#if defined(MAP_STACK)
+#if defined(MAP_STACK) &&	\
+    !defined(__FreeBSD__)
 	MAP_STACK,
 #endif
 #if defined(MAP_EXECUTABLE)
@@ -194,6 +232,10 @@ static const int mmap_flags[] = {
 #if defined(MAP_PREFAULT_READ)
 	MAP_PREFAULT_READ,
 #endif
+/* FreeBSD */
+#if defined(MAP_ALIGNED_SUPER)
+	MAP_ALIGNED_SUPER,
+#endif
 /* NetBSD */
 #if defined(MAP_WIRED)
 	MAP_WIRED,
@@ -225,8 +267,10 @@ static void MLOCKED_TEXT stress_mmap_sighandler(int signum)
 {
 	(void)signum;
 
-	if (jmp_env_set)
+	if (jmp_env_set) {
 		siglongjmp(jmp_env, 1);
+		stress_no_return();
+	}
 }
 
 /*
@@ -331,7 +375,7 @@ static void stress_mmap_invalid(
 
 	ptr = mmap(addr, length, prot, flags, fd, offset);
 	if (UNLIKELY(ptr != MAP_FAILED))
-		(void)stress_munmap_retry_enomem(ptr, length);
+		(void)stress_munmap_force(ptr, length);
 
 #if defined(__NR_mmap) &&	\
     defined(HAVE_SYSCALL)
@@ -341,12 +385,12 @@ static void stress_mmap_invalid(
 	 */
 	ptr = (void *)(uintptr_t)syscall(__NR_mmap, addr, length, prot, flags, fd, offset + 1);
 	if (UNLIKELY(ptr != MAP_FAILED))
-		(void)stress_munmap_retry_enomem(ptr, length);
+		(void)stress_munmap_force(ptr, length);
 #endif
 	/* Do the above via libc */
 	ptr = mmap(addr, length, prot, flags, fd, offset + 1);
 	if (UNLIKELY(ptr != MAP_FAILED))
-		(void)stress_munmap_retry_enomem(ptr, length);
+		(void)stress_munmap_force(ptr, length);
 }
 
 /*
@@ -416,14 +460,14 @@ static void stress_mmap_fast_munmap(
 			if (mappings[i] == munmap_start + munmap_size) {
 				munmap_size += page_size;
 			} else {
-				(void)stress_munmap_retry_enomem((void *)munmap_start, munmap_size);
+				(void)stress_munmap_force((void *)munmap_start, munmap_size);
 				munmap_start = mappings[i];
 				munmap_size = page_size;
 			}
 		}
 	}
 	if (munmap_start && (munmap_size > 0))
-		(void)stress_munmap_retry_enomem((void *)munmap_start, munmap_size);
+		(void)stress_munmap_force((void *)munmap_start, munmap_size);
 	(void)shim_memset(mapped, 0, pages);
 }
 
@@ -441,7 +485,7 @@ static void stress_mmap_slow_munmap(
 
 	for (i = 0; i < pages; i++) {
 		if (mapped[i] == PAGE_MAPPED)
-			(void)stress_munmap_retry_enomem((void *)mappings[i], page_size);
+			(void)stress_munmap_force((void *)mappings[i], page_size);
 	}
 	(void)shim_memset(mapped, 0, pages);
 }
@@ -456,7 +500,10 @@ static int stress_mmap_child(stress_args_t *args, void *ctxt)
 	const int fd = context->fd;
 	NOCLOBBER int no_mem_retries = 0;
 	const int bad_fd = stress_get_bad_fd();
+#if defined(MS_ASYNC) &&	\
+    defined(MS_SYNC)
 	const int ms_flags = context->mmap_async ? MS_ASYNC : MS_SYNC;
+#endif
 	uint8_t *mapped, **mappings;
 	size_t *idx;
 	void *hint;
@@ -481,8 +528,10 @@ static int stress_mmap_child(stress_args_t *args, void *ctxt)
 				PROT_READ | PROT_WRITE,
 				MAP_ANONYMOUS | MAP_PRIVATE, -1, 0);
 	if (mappings == MAP_FAILED) {
-		pr_dbg("%s: cannot allocate mappings buffer, errno=%d (%s)\n",
-			args->name, errno, strerror(errno));
+		pr_dbg("%s: cannot allocate %zu byte mappings buffer%s, errno=%d (%s)\n",
+			args->name, pages * sizeof(*mappings),
+			stress_get_memfree_str(),
+			errno, strerror(errno));
 		(void)munmap((void *)mapped, pages * sizeof(*mapped));
 		return EXIT_NO_RESOURCE;
 	}
@@ -494,8 +543,9 @@ static int stress_mmap_child(stress_args_t *args, void *ctxt)
 				PROT_READ | PROT_WRITE,
 				MAP_ANONYMOUS | MAP_PRIVATE, -1, 0);
 	if (idx == MAP_FAILED) {
-		pr_dbg("%s: cannot allocate idx buffer, errno=%d (%s)\n",
-			args->name, errno, strerror(errno));
+		pr_dbg("%s: cannot allocate %zu byte idx buffer%s, errno=%d (%s)\n",
+			args->name, pages * sizeof(*idx),
+			stress_get_memfree_str(), errno, strerror(errno));
 		(void)munmap((void *)mappings, pages * sizeof(*mappings));
 		(void)munmap((void *)mapped, pages * sizeof(*mapped));
 		return EXIT_NO_RESOURCE;
@@ -549,6 +599,7 @@ retry:
 		hint = stress_mwc1() ? NULL : (void *)~(uintptr_t)0;
 		buf = (uint8_t *)context->mmap(hint, sz,
 			PROT_READ | PROT_WRITE, (context->flags | rnd_flag) & mask, fd, 0);
+
 		if (buf == MAP_FAILED) {
 #if defined(MAP_POPULATE)
 			/* Force MAP_POPULATE off, just in case */
@@ -558,14 +609,6 @@ retry:
 				continue;
 			}
 #endif
-#if defined(MAP_HUGETLB)
-			/* Force MAP_HUGETLB off, just in case */
-			if (rnd_flag & MAP_HUGETLB) {
-				mask &= ~MAP_HUGETLB;
-				no_mem_retries++;
-				goto retry;
-			}
-#endif
 #if defined(MAP_UNINITIALIZED)
 			/* Force MAP_UNINITIALIZED off, just in case */
 			if (rnd_flag & MAP_UNINITIALIZED) {
@@ -593,13 +636,16 @@ retry:
 		no_mem_retries = 0;
 		if (mmap_file) {
 			(void)shim_memset(buf, 0xff, sz);
+#if defined(MS_ASYNC) &&	\
+    defined(MS_SYNC)
 			(void)shim_msync((void *)buf, sz, ms_flags);
+#endif
 		}
 		if (context->mmap_madvise)
-			(void)stress_madvise_random(buf, sz);
+			(void)stress_madvise_randomize(buf, sz);
 #if defined(HAVE_LINUX_MEMPOLICY_H)
 		if (context->mmap_numa)
-			stress_numa_randomize_pages(context->numa_mask, buf, page_size, sz);
+			stress_numa_randomize_pages(args, context->numa_nodes, context->numa_mask, buf, sz, page_size);
 #endif
 		if (context->mmap_mergeable)
 			(void)stress_madvise_mergeable(buf, sz);
@@ -660,7 +706,7 @@ retry:
 				}
 #endif
 				if (context->mmap_madvise)
-					(void)stress_madvise_random(mappings[page], page_size);
+					(void)stress_madvise_randomize(mappings[page], page_size);
 				stress_mmap_mprotect(args->name, mappings[page],
 					page_size, page_size, context->mmap_mprotect);
 			}
@@ -675,7 +721,7 @@ retry:
 		else
 			stress_mmap_fast_munmap(mappings, mapped, pages, page_size);
 
-		(void)stress_munmap_retry_enomem((void *)buf, sz);
+		(void)stress_munmap_force((void *)buf, sz);
 #if defined(MAP_FIXED)
 
 		/*
@@ -710,7 +756,7 @@ retry:
 						(void)shim_mlock(mappings[page], page_size);
 					(void)stress_mincore_touch_pages(mappings[page], page_size);
 					if (context->mmap_madvise)
-						(void)stress_madvise_random(mappings[page], page_size);
+						(void)stress_madvise_randomize(mappings[page], page_size);
 					if (context->mmap_mergeable)
 						(void)stress_madvise_mergeable(mappings[page], page_size);
 					stress_mmap_mprotect(args->name, mappings[page],
@@ -727,7 +773,10 @@ retry:
 					}
 					if (mmap_file) {
 						(void)shim_memset(mappings[page], (int)n, page_size);
+#if defined(MS_ASYNC) &&	\
+    defined(MS_SYNC)
 						(void)shim_msync((void *)mappings[page], page_size, ms_flags);
+#endif
 #if defined(FALLOC_FL_KEEP_SIZE) &&	\
     defined(FALLOC_FL_PUNCH_HOLE)
 						(void)shim_fallocate(fd, FALLOC_FL_PUNCH_HOLE | FALLOC_FL_KEEP_SIZE,
@@ -756,8 +805,8 @@ cleanup:
 		 */
 		for (n = 0; n < pages; n++) {
 			if (mapped[n] & PAGE_MAPPED) {
-				(void)stress_munmap_retry_enomem((void *)mappings[n], 0);
-				(void)stress_munmap_retry_enomem((void *)mappings[n], page_size);
+				(void)stress_munmap_force((void *)mappings[n], 0);
+				(void)stress_munmap_force((void *)mappings[n], page_size);
 				break;
 			}
 		}
@@ -800,7 +849,7 @@ cleanup:
 			if (buf != MAP_FAILED) {
 				if (context->mmap_mlock)
 					(void)shim_mlock(buf, rnd_sz);
-				(void)stress_munmap_retry_enomem((void *)buf, rnd_sz);
+				(void)stress_munmap_force((void *)buf, rnd_sz);
 			}
 		}
 
@@ -822,7 +871,7 @@ cleanup:
 				if (context->mmap_mlock)
 					(void)shim_mlock(buf, page_size);
 				stress_set_vma_anon_name((void *)buf, page_size, mmap_name);
-				(void)stress_munmap_retry_enomem((void *)buf, page_size);
+				(void)stress_munmap_force((void *)buf, page_size);
 			}
 			if (tmpfd >= 0)
 				(void)close(tmpfd);
@@ -862,7 +911,7 @@ cleanup:
 					rc = EXIT_FAILURE;
 				}
 			}
-			(void)stress_munmap_retry_enomem((void *)buf64, page_size);
+			(void)stress_munmap_force((void *)buf64, page_size);
 		}
 #endif
 #if defined(HAVE_MPROTECT)
@@ -886,7 +935,7 @@ cleanup:
 					rc = EXIT_FAILURE;
 				}
 			}
-			(void)stress_munmap_retry_enomem((void *)buf64, page_size);
+			(void)stress_munmap_force((void *)buf64, page_size);
 		}
 #endif
 		stress_bogo_inc(args);
@@ -914,7 +963,7 @@ static int stress_mmap(stress_args_t *args)
 	bool mmap_mmap2 = false;
 	int ret, all_flags;
 	stress_mmap_context_t context;
-	size_t i;
+	size_t i, mmap_total;
 
 	jmp_env_set = false;
 
@@ -928,12 +977,15 @@ static int stress_mmap(stress_args_t *args)
 	context.mmap_mlock = false;
 	context.mmap_mprotect = false;
 	context.mmap_numa = false;
+	context.mmap_slow_munmap = false;
+	context.mmap_write_check = false;
 	context.flags = MAP_PRIVATE | MAP_ANONYMOUS;
 #if defined(MAP_POPULATE)
 	context.flags |= MAP_POPULATE;
 #endif
 #if defined(HAVE_LINUX_MEMPOLICY_H)
 	context.numa_mask = NULL;
+	context.numa_nodes = NULL;
 #endif
 
 	(void)stress_get_setting("mmap-async", &context.mmap_async);
@@ -968,24 +1020,31 @@ static int stress_mmap(stress_args_t *args)
     defined(__NR_mmap2)
 		context.mmap = (mmap_func_t)mmap2_try;
 #else
-		if (args->instance == 0)
+		if (stress_instance_zero(args))
 			pr_inf("%s: using mmap instead of mmap2 as it is not available\n",
 				args->name);
 #endif
 	}
 
-	if (!stress_get_setting("mmap-bytes", &context.mmap_bytes)) {
+	mmap_total = DEFAULT_MMAP_BYTES;
+	if (!stress_get_setting("mmap-bytes", &mmap_total)) {
 		if (g_opt_flags & OPT_FLAGS_MAXIMIZE)
-			context.mmap_bytes = MAX_32;
+			mmap_total = MAX_32;
 		if (g_opt_flags & OPT_FLAGS_MINIMIZE)
-			context.mmap_bytes = MIN_MMAP_BYTES;
+			mmap_total = MIN_MMAP_BYTES;
 	}
-	context.mmap_bytes /= args->instances;
-	if (context.mmap_bytes < MIN_MMAP_BYTES)
+	context.mmap_bytes = mmap_total / args->instances;
+	if (context.mmap_bytes < MIN_MMAP_BYTES) {
 		context.mmap_bytes = MIN_MMAP_BYTES;
-	if (context.mmap_bytes < page_size)
+		mmap_total = context.mmap_bytes * args->instances;
+	}
+	if (context.mmap_bytes < page_size) {
 		context.mmap_bytes = page_size;
+		mmap_total = context.mmap_bytes * args->instances;
+	}
 	context.sz = context.mmap_bytes & ~(page_size - 1);
+	if (stress_instance_zero(args))
+		stress_usage_bytes(args, context.mmap_bytes, mmap_total);
 
 	if (context.mmap_file) {
 		int file_flags = O_CREAT | O_RDWR;
@@ -1060,21 +1119,11 @@ redo:
 
 	if (context.mmap_numa) {
 #if defined(HAVE_LINUX_MEMPOLICY_H)
-		if (stress_numa_nodes() > 1) {
-			context.numa_mask = stress_numa_mask_alloc();
-			if (!context.numa_mask) {
-				pr_inf("%s: cannot allocate NUMA mask, disabling --mmap-numa\n",
-					args->name);
-				context.mmap_numa = false;
-			}
-		} else {
-			if (args->instance == 0)
-				pr_inf("%s: only 1 NUMA node available, disabling --mmap-numa\n",
-					args->name);
-			context.mmap_numa = false;
-		}
+		stress_numa_mask_and_node_alloc(args, &context.numa_nodes,
+						&context.numa_mask, "--mmap-numa",
+						&context.mmap_numa);
 #else
-		if (args->instance == 0)
+		if (stress_instance_zero(args))
 			pr_inf("%s: --mmap-numa selected but not supported by this system, disabling option\n",
 				args->name);
 		context.mmap_numa = false;
@@ -1098,51 +1147,32 @@ redo:
 	if (context.mmap_flag_perms)
 		free(context.mmap_flag_perms);
 #if defined(HAVE_LINUX_MEMPOLICY_H)
-	if (context.mmap_numa)
+	if (context.mmap_numa) {
 		stress_numa_mask_free(context.numa_mask);
+		stress_numa_mask_free(context.numa_nodes);
+	}
 #endif
 
 	return ret;
 }
 
-static void stress_mmap_stressful(const char *opt_name, const char *opt_arg, stress_type_id_t *type_id, void *value)
-{
-	(void)opt_name;
-	*type_id = TYPE_ID_SIZE_T;
-	*(size_t *)value = 0;
-
-	(void)stress_set_setting_true("mmap", "mmap-mergeable", opt_arg);
-	(void)stress_set_setting_true("mmap", "mmap-mprotect", opt_arg);
-	(void)stress_set_setting_true("mmap", "mmap-file", opt_arg);
-	(void)stress_set_setting_true("mmap", "mmap-odirect", opt_arg);
-	(void)stress_set_setting_true("mmap", "mmap-madvise", opt_arg);
-	(void)stress_set_setting_true("mmap", "mmap-mlock", opt_arg);
-	(void)stress_set_setting_true("mmap", "mmap-numa", opt_arg);
-	(void)stress_set_setting_true("mmap", "mmap-slow-munmap", opt_arg);
-}
-
-static const stress_opt_t opts[] = {
-	{ OPT_mmap_async,       "mmap-async",       TYPE_ID_BOOL, 0, 1, NULL },
-	{ OPT_mmap_bytes,       "mmap-bytes",       TYPE_ID_SIZE_T_BYTES_VM, MIN_MMAP_BYTES, MAX_MMAP_BYTES, NULL },
-	{ OPT_mmap_file,        "mmap-file",        TYPE_ID_BOOL, 0, 1, NULL },
-	{ OPT_mmap_madvise,     "mmap-madvise",     TYPE_ID_BOOL, 0, 1, NULL },
-	{ OPT_mmap_mergeable,   "mmap-mergeable",   TYPE_ID_BOOL, 0, 1, NULL },
-	{ OPT_mmap_mlock,       "mmap-mlock",       TYPE_ID_BOOL, 0, 1, NULL },
-	{ OPT_mmap_mmap2,       "mmap-mmap2",       TYPE_ID_BOOL, 0, 1, NULL },
-	{ OPT_mmap_mprotect,    "mmap-mprotect",    TYPE_ID_BOOL, 0, 1, NULL },
-	{ OPT_mmap_numa,	"mmap-numa",	    TYPE_ID_BOOL, 0, 1, NULL },
-	{ OPT_mmap_odirect,     "mmap-odirect",     TYPE_ID_BOOL, 0, 1, NULL },
-	{ OPT_mmap_osync,       "mmap-osync",       TYPE_ID_BOOL, 0, 1, NULL },
-	{ OPT_mmap_slow_munmap,	"mmap-slow-munmap", TYPE_ID_BOOL, 0, 1, NULL },
-	{ OPT_mmap_stressful,   "mmap-stressful",   TYPE_ID_CALLBACK, 0, 0, stress_mmap_stressful },
-	{ OPT_mmap_write_check, "mmap-write-check", TYPE_ID_BOOL, 0, 1, NULL },
-	END_OPT,
-};
-
 const stressor_info_t stress_mmap_info = {
 	.stressor = stress_mmap,
-	.class = CLASS_VM | CLASS_OS,
+	.classifier = CLASS_VM | CLASS_OS,
 	.opts = opts,
 	.verify = VERIFY_OPTIONAL,
 	.help = help
 };
+
+#else
+
+const stressor_info_t stress_mmap_info = {
+	.stressor = stress_unimplemented,
+	.classifier = CLASS_VM | CLASS_OS,
+	.opts = opts,
+	.verify = VERIFY_OPTIONAL,
+	.help = help,
+	.unimplemented_reason = "built without siglongjmp support"
+};
+
+#endif
