@@ -19,6 +19,7 @@
  */
 #include "stress-ng.h"
 #include "core-builtin.h"
+#include "core-mmap.h"
 #include "core-out-of-memory.h"
 
 #define MIN_MSYNC_BYTES		(1 * MB)  /* MUST NOT BE page size or less! */
@@ -38,7 +39,7 @@ static const stress_help_t help[] = {
 };
 
 static const stress_opt_t opts[] = {
-	{ OPT_msync_bytes, "msync-bytes", TYPE_ID_SIZE_T_BYTES_FS, MIN_MSYNC_BYTES, MAX_MSYNC_BYTES, NULL },
+	{ OPT_msync_bytes, "msync-bytes", TYPE_ID_UINT64_BYTES_FS, MIN_MSYNC_BYTES, MAX_MSYNC_BYTES, NULL },
 	END_OPT,
 };
 
@@ -89,6 +90,7 @@ static void MLOCKED_TEXT NORETURN stress_sigbus_handler(int signum)
 	sigbus_count++;
 
 	siglongjmp(jmp_env, 1); /* bounce back */
+	stress_no_return();
 }
 
 /*
@@ -101,8 +103,8 @@ static int stress_msync(stress_args_t *args)
 	uint8_t *data = NULL;
 	const size_t page_size = args->page_size;
 	const size_t min_size = 2 * page_size;
-	size_t msync_bytes = DEFAULT_MSYNC_BYTES;
-	NOCLOBBER size_t sz;
+	uint64_t msync_bytes, msync_bytes_total = DEFAULT_MSYNC_BYTES;
+	NOCLOBBER uint64_t sz;
 	ssize_t ret;
 	NOCLOBBER ssize_t rc = EXIT_SUCCESS;
 	NOCLOBBER int fd = -1;
@@ -116,20 +118,35 @@ static int stress_msync(stress_args_t *args)
 	if (stress_sighandler(args->name, SIGBUS, stress_sigbus_handler, NULL) < 0)
 		return EXIT_FAILURE;
 
-	if (!stress_get_setting("msync-bytes", &msync_bytes)) {
+	if (!stress_get_setting("msync-bytes", &msync_bytes_total)) {
 		if (g_opt_flags & OPT_FLAGS_MAXIMIZE)
-			msync_bytes = MAXIMIZED_FILE_SIZE;
+			msync_bytes_total = MAXIMIZED_FILE_SIZE;
 		if (g_opt_flags & OPT_FLAGS_MINIMIZE)
-			msync_bytes = MIN_MSYNC_BYTES;
+			msync_bytes_total = MIN_MSYNC_BYTES;
 	}
-	msync_bytes /= args->instances;
+	if (msync_bytes_total < MIN_MSYNC_BYTES) {
+		msync_bytes_total = MIN_MSYNC_BYTES;
+		if (stress_instance_zero(args))
+			pr_inf("%s: --msync-bytes too small, using %" PRIu64 " instead\n",
+				args->name, msync_bytes_total);
+	}
+	if (msync_bytes_total > MAX_MSYNC_BYTES) {
+		msync_bytes_total = MAX_MSYNC_BYTES;
+		if (stress_instance_zero(args))
+			pr_inf("%s: --msync-bytes too large, using %" PRIu64 " instead\n",
+				args->name, msync_bytes_total);
+	}
+
+	msync_bytes = msync_bytes_total / args->instances;
 	if (msync_bytes < MIN_MSYNC_BYTES)
 		msync_bytes = MIN_MSYNC_BYTES;
 	if (msync_bytes < page_size)
 		msync_bytes = page_size;
+	if (stress_instance_zero(args))
+		stress_usage_bytes(args, msync_bytes, msync_bytes_total);
 	sz = msync_bytes & ~(page_size - 1);
-	if (sz < min_size)
-		sz = min_size;
+	if (sz < (uint64_t)min_size)
+		sz = (uint64_t)min_size;
 
 	/* Make sure this is killable by OOM killer */
 	stress_set_oom_adjustment(args, true);
@@ -164,16 +181,18 @@ static int stress_msync(stress_args_t *args)
 	buf = (uint8_t *)stress_mmap_populate(NULL, sz,
 		PROT_READ | PROT_WRITE, MAP_SHARED, fd, 0);
 	if (buf == MAP_FAILED) {
-		pr_err("%s: failed to mmap memory of size %zu, errno=%d (%s)\n",
-			args->name, sz, errno, strerror(errno));
+		pr_err("%s: failed to mmap memory of size %" PRIu64 " bytes%s, errno=%d (%s)\n",
+			args->name, sz,
+			stress_get_memfree_str(), errno, strerror(errno));
 		rc = EXIT_NO_RESOURCE;
 		goto err;
 	}
 	data = (uint8_t *)stress_mmap_populate(NULL, page_size,
 		PROT_READ | PROT_WRITE, MAP_SHARED | MAP_ANONYMOUS, -1, 0);
 	if (data == MAP_FAILED) {
-		pr_err("%s: failed to mmap memory of size %zu, errno=%d (%s)\n",
-			args->name, page_size, errno, strerror(errno));
+		pr_err("%s: failed to mmap memory of size %zu bytes%s, errno=%d (%s)\n",
+			args->name, page_size,
+			stress_get_memfree_str(), errno, strerror(errno));
 		rc = EXIT_NO_RESOURCE;
 		goto err_unmap;
 	}
@@ -307,7 +326,7 @@ err:
 
 const stressor_info_t stress_msync_info = {
 	.stressor = stress_msync,
-	.class = CLASS_VM | CLASS_OS,
+	.classifier = CLASS_VM | CLASS_OS,
 	.opts = opts,
 	.verify = VERIFY_ALWAYS,
 	.help = help
@@ -315,7 +334,7 @@ const stressor_info_t stress_msync_info = {
 #else
 const stressor_info_t stress_msync_info = {
 	.stressor = stress_unimplemented,
-	.class = CLASS_VM | CLASS_OS,
+	.classifier = CLASS_VM | CLASS_OS,
 	.opts = opts,
 	.verify = VERIFY_ALWAYS,
 	.help = help,
