@@ -45,7 +45,7 @@ static const stress_help_t help[] = {
 static const stress_opt_t opts[] = {
 	{ OPT_numa_bytes,        "numa-bytes",        TYPE_ID_SIZE_T_BYTES_VM, MIN_NUMA_MMAP_BYTES, MAX_NUMA_MMAP_BYTES, NULL },
 	{ OPT_numa_shuffle_addr, "numa-shuffle-addr", TYPE_ID_BOOL, 0, 1, NULL },
-	{ OPT_numa_shuffle_node, "numa-shiffle-node", TYPE_ID_BOOL, 0, 1, NULL },
+	{ OPT_numa_shuffle_node, "numa-shuffle-node", TYPE_ID_BOOL, 0, 1, NULL },
 	END_OPT,
 };
 
@@ -177,7 +177,8 @@ static void stress_numa_check_maps(
 static int stress_numa(stress_args_t *args)
 {
 	const size_t page_size = args->page_size;
-	size_t num_pages, numa_bytes = 0;
+	size_t num_pages, numa_bytes, numa_bytes_total = DEFAULT_NUMA_MMAP_BYTES;
+
 	uint8_t *buf;
 	int rc = EXIT_FAILURE;
 	const bool cap_sys_nice = stress_check_capability(SHIM_CAP_SYS_NICE);
@@ -193,11 +194,11 @@ static int stress_numa(stress_args_t *args)
 	stress_numa_mask_t *numa_mask, *old_numa_mask, *numa_nodes;
 	unsigned long int node;
 
-	if (!stress_get_setting("numa-bytes", &numa_bytes)) {
+	if (!stress_get_setting("numa-bytes", &numa_bytes_total)) {
 		if (g_opt_flags & OPT_FLAGS_MAXIMIZE)
-			numa_bytes = MAX_32;
+			numa_bytes_total = MAX_32;
 		if (g_opt_flags & OPT_FLAGS_MINIMIZE)
-			numa_bytes = MIN_NUMA_MMAP_BYTES;
+			numa_bytes_total = MIN_NUMA_MMAP_BYTES;
 	}
 	if (!stress_get_setting("numa-shuffle-addr", &numa_shuffle_addr)) {
 		if (g_opt_flags & OPT_FLAGS_AGGRESSIVE)
@@ -208,39 +209,35 @@ static int stress_numa(stress_args_t *args)
 			numa_shuffle_node = true;
 	}
 
-	if (numa_bytes == 0) {
-		numa_bytes = DEFAULT_NUMA_MMAP_BYTES;
-	} else {
-		if (args->instances > 0) {
-			numa_bytes /= args->instances;
-			numa_bytes &= ~(page_size - 1);
-		}
-		if (numa_bytes < MIN_NUMA_MMAP_BYTES)
-			numa_bytes = MIN_NUMA_MMAP_BYTES;
-	}
+	numa_bytes = numa_bytes_total / args->instances;
+	numa_bytes &= ~(page_size - 1);
+	if (numa_bytes < MIN_NUMA_MMAP_BYTES)
+		numa_bytes = MIN_NUMA_MMAP_BYTES;
+	if (stress_instance_zero(args))
+		stress_usage_bytes(args, numa_bytes, numa_bytes_total);
 
 	num_pages = numa_bytes / page_size;
 
 	numa_nodes = stress_numa_mask_alloc();
 	if (!numa_nodes) {
-		pr_inf_skip("%s: no NUMA nodes found, skipping test\n", args->name);
+		pr_inf_skip("%s: no NUMA nodes found, skipping stressor\n", args->name);
 		rc = EXIT_NO_RESOURCE;
 		goto deinit;
 	}
 	if (stress_numa_mask_nodes_get(numa_nodes) < 1) {
-		pr_inf_skip("%s: no NUMA nodes found, skipping test\n", args->name);
+		pr_inf_skip("%s: no NUMA nodes found, skipping stressor\n", args->name);
 		rc = EXIT_NO_RESOURCE;
 		goto numa_nodes_free;
 	}
 	numa_mask = stress_numa_mask_alloc();
 	if (!numa_mask) {
-		pr_inf_skip("%s: no NUMA nodes found, skipping test\n", args->name);
+		pr_inf_skip("%s: no NUMA nodes found, skipping stressor\n", args->name);
 		rc = EXIT_NO_RESOURCE;
 		goto numa_nodes_free;
 	}
 	old_numa_mask = stress_numa_mask_alloc();
 	if (!old_numa_mask) {
-		pr_inf_skip("%s: no NUMA nodes found, skipping test\n", args->name);
+		pr_inf_skip("%s: no NUMA nodes found, skipping stressor\n", args->name);
 		rc = EXIT_NO_RESOURCE;
 		goto numa_mask_free;
 	}
@@ -258,8 +255,10 @@ static int stress_numa(stress_args_t *args)
 					PROT_READ | PROT_WRITE,
 					MAP_ANONYMOUS | MAP_SHARED, 0, 0);
 	if (status == MAP_FAILED) {
-		pr_inf_skip("%s: cannot mmap status array of %zu elements, skipping stressor\n",
-			args->name, num_pages);
+		pr_inf_skip("%s: failed to mmap status array of %zu elements%s, "
+			"errno=%d (%s), skipping stressor\n",
+			args->name, num_pages,
+			stress_get_memfree_str(), errno, strerror(errno));
 		rc = EXIT_NO_RESOURCE;
 		goto old_numa_mask_free;
 	}
@@ -270,8 +269,10 @@ static int stress_numa(stress_args_t *args)
 					PROT_READ | PROT_WRITE,
 					MAP_ANONYMOUS | MAP_SHARED, 0, 0);
 	if (dest_nodes == MAP_FAILED) {
-		pr_inf("%s: cannot mmap dest_nodes array of %zu elements, skipping stressor\n",
-			args->name, num_pages);
+		pr_inf("%s: failed to mmap dest_nodes array of %zu elements%s, "
+			"errno=%d (%s), skipping stressor\n",
+			args->name, num_pages,
+			stress_get_memfree_str(), errno, strerror(errno));
 		rc = EXIT_NO_RESOURCE;
 		goto status_free;
 	}
@@ -282,8 +283,10 @@ static int stress_numa(stress_args_t *args)
 					PROT_READ | PROT_WRITE,
 					MAP_ANONYMOUS | MAP_SHARED, 0, 0);
 	if (pages == MAP_FAILED) {
-		pr_inf_skip("%s: cannot mmap pages array of %zu elements, skipping stressor\n",
-			args->name, num_pages);
+		pr_inf_skip("%s: failed to mmap pages array of %zu elements%s, "
+			"errno=%d (%s), skipping stressor\n",
+			args->name, num_pages,
+			stress_get_memfree_str(), errno, strerror(errno));
 		rc = EXIT_NO_RESOURCE;
 		goto dest_nodes_free;
 	}
@@ -296,9 +299,11 @@ static int stress_numa(stress_args_t *args)
 		PROT_READ | PROT_WRITE,
 		MAP_ANONYMOUS | MAP_SHARED, 0, 0);
 	if (buf == MAP_FAILED) {
-		rc = stress_exit_status(errno);
-		pr_fail("%s: mmap'd region of %zu bytes failed\n",
-			args->name, numa_bytes);
+		pr_inf_skip("%s: failed to mmap a region of %zu bytes%s, "
+			"errno=%d (%s), skipping stressor\n",
+			args->name, numa_bytes,
+			stress_get_memfree_str(), errno, strerror(errno));
+		rc = EXIT_NO_RESOURCE;
 		goto pages_free;
 	}
 	stress_set_vma_anon_name(buf, numa_bytes, "numa-shared-data");
@@ -362,7 +367,7 @@ static int stress_numa(stress_args_t *args)
 		if (UNLIKELY(!stress_continue_flag()))
 			break;
 
-		ret = shim_set_mempolicy(MPOL_PREFERRED, NULL, numa_mask->max_nodes);
+		ret = shim_set_mempolicy(MPOL_PREFERRED, NULL, 0);
 		if (UNLIKELY(ret < 0)) {
 			if (errno != ENOSYS) {
 				pr_fail("%s: set_mempolicy failed, errno=%d (%s)\n",
@@ -753,7 +758,7 @@ deinit:
 
 const stressor_info_t stress_numa_info = {
 	.stressor = stress_numa,
-	.class = CLASS_CPU | CLASS_MEMORY | CLASS_OS,
+	.classifier = CLASS_CPU | CLASS_MEMORY | CLASS_OS,
 	.verify = VERIFY_ALWAYS,
 	.opts = opts,
 	.help = help
@@ -761,7 +766,7 @@ const stressor_info_t stress_numa_info = {
 #else
 const stressor_info_t stress_numa_info = {
 	.stressor = stress_unimplemented,
-	.class = CLASS_CPU | CLASS_MEMORY | CLASS_OS,
+	.classifier = CLASS_CPU | CLASS_MEMORY | CLASS_OS,
 	.verify = VERIFY_ALWAYS,
 	.opts = opts,
 	.help = help,
