@@ -219,13 +219,66 @@ static void stress_proc_self_mem(stress_args_t *args, const int fd)
 static void stress_proc_mtrr(stress_args_t *args, const int fd)
 {
 	struct mtrr_gentry gentry;
+#if defined(HAVE_MTRR_SENTRY)
+	struct mtrr_sentry sentry;
+#endif
+#if defined(HAVE_MTRR_SENTRY)
+	const uint32_t base = stress_mwc32() & ~(4095U);
+	bool base_unique = true;
+#endif
 
 	(void)args;
 
 	(void)shim_memset(&gentry, 0, sizeof(gentry));
 	while (ioctl(fd, MTRRIOC_GET_ENTRY, &gentry) == 0) {
+#if defined(HAVE_MTRR_SENTRY)
+		if (base == gentry.base)
+			base_unique = false;
+#endif
 		gentry.regnum++;
+
+#if defined(HAVE_MTRR_SENTRY) &&	\
+    defined(MTRRIOC_SET_ENTRY)
+		(void)memset(&sentry, 0, sizeof(sentry));
+		sentry.base = gentry.base;
+		sentry.size = 0;
+		sentry.type = gentry.type;
+		VOID_RET(int, ioctl(fd, MTRRIOC_SET_ENTRY, &sentry));
+
+		(void)memset(&sentry, 0, sizeof(sentry));
+		sentry.base = gentry.base;
+		sentry.size = gentry.size;
+		sentry.type = ~0;
+		VOID_RET(int, ioctl(fd, MTRRIOC_SET_ENTRY, &sentry));
+
+		(void)memset(&sentry, 0, sizeof(sentry));
+		sentry.base = gentry.base;
+		sentry.size = ~4095;
+		sentry.type = gentry.type;
+		VOID_RET(int, ioctl(fd, MTRRIOC_SET_ENTRY, &sentry));
+#endif
 	}
+
+#if defined(HAVE_MTRR_SENTRY)
+	if (!base_unique)
+		return;
+
+#if defined(MTRRIOC_DEL_ENTRY)
+	/* delete non-existent mtrr, will fail */
+	sentry.base = base;
+	sentry.size = 4096;
+	sentry.type = 1;
+	VOID_RET(int, ioctl(fd, MTRRIOC_DEL_ENTRY, &sentry));
+#endif
+
+#if defined(MTRRIOC_KILL_ENTRY)
+	/* kill non-existent mtrr, will fail */
+	sentry.base = base;
+	sentry.size = 4096;
+	sentry.type = 1;
+	VOID_RET(int, ioctl(fd, MTRRIOC_KILL_ENTRY, &sentry));
+#endif
+#endif
 }
 #endif
 
@@ -249,7 +302,7 @@ static void stress_proc_pci(stress_args_t *args, const int fd)
 
 #endif
 
-static stress_proc_info_t stress_proc_info[] = {
+static const stress_proc_info_t stress_proc_info[] = {
 	{ "/proc/self/mem",		stress_proc_self_mem },
 #if defined(HAVE_ASM_MTRR_H) &&		\
     defined(HAVE_MTRR_GENTRY) &&	\
@@ -297,11 +350,20 @@ static inline void stress_proc_rw(
 redo:
 		if (UNLIKELY(!*path || !stress_continue_flag()))
 			break;
-
 		if (!strncmp(path, "/proc/self", 10))
 			procfs_flag &= ~PROCFS_FLAG_WRITE;
 		if (!strncmp(path, "/proc/", 6) && isdigit((unsigned char)path[6]))
 			procfs_flag &= ~PROCFS_FLAG_WRITE;
+#if defined(__CYGWIN__)
+		/*
+		 *  Concurrent access on /proc/$PID/maps and /proc/$PID/ctty
+		 *  on Cygwin causes issues (Jul 2025), so skip these
+		 */
+		if ((!strncmp(path, "/proc/", 6) && isdigit((unsigned char)path[6]))) {
+			if (strstr(path, "maps") || strstr(path, "ctty"))
+				return;
+		}
+#endif
 
 		t_start = stress_time_now();
 		if ((fd = open(path, O_RDONLY | O_NONBLOCK)) < 0)
@@ -816,7 +878,7 @@ static char *stress_random_pid(void)
  */
 static int stress_procfs_no_entries(stress_args_t *args)
 {
-	if (args->instance == 0)
+	if (stress_instance_zero(args))
 		pr_inf_skip("%s: no /proc entries found, skipping stressor\n", args->name);
 	return EXIT_NO_RESOURCE;
 }
@@ -924,13 +986,13 @@ static int stress_procfs(stress_args_t *args)
 
 const stressor_info_t stress_procfs_info = {
 	.stressor = stress_procfs,
-	.class = CLASS_FILESYSTEM | CLASS_OS,
+	.classifier = CLASS_FILESYSTEM | CLASS_OS,
 	.help = help
 };
 #else
 const stressor_info_t stress_procfs_info = {
 	.stressor = stress_unimplemented,
-	.class = CLASS_FILESYSTEM | CLASS_OS,
+	.classifier = CLASS_FILESYSTEM | CLASS_OS,
 	.help = help,
 	.unimplemented_reason = "built without librt or only supported on Linux"
 };
