@@ -115,7 +115,6 @@ static const int normal_policies[] = {
 
 static uint32_t stress_call_race_sched_method_idx(const uint32_t cpu_idx, size_t method_index)
 {
-
 	static size_t method_all_index = 1;
 	uint32_t new_cpu_idx = cpu_idx;
 
@@ -456,7 +455,7 @@ static int stress_race_sched(stress_args_t *args)
 
 const stressor_info_t stress_race_sched_info = {
 	.stressor = stress_race_sched,
-	.class = CLASS_SCHEDULER | CLASS_OS,
+	.classifier = CLASS_SCHEDULER | CLASS_OS,
 	.opts = opts,
 	.verify = VERIFY_ALWAYS,
 	.help = help
@@ -466,7 +465,7 @@ const stressor_info_t stress_race_sched_info = {
 
 const stressor_info_t stress_race_sched_info = {
         .stressor = stress_unimplemented,
-	.class = CLASS_SCHEDULER | CLASS_OS,
+	.classifier = CLASS_SCHEDULER | CLASS_OS,
 	.opts = opts,
 	.help = help,
 	.verify = VERIFY_ALWAYS,
