@@ -19,6 +19,7 @@
 #include "stress-ng.h"
 #include "core-builtin.h"
 #include "core-killpid.h"
+#include "core-mmap.h"
 
 #include <sched.h>
 
@@ -60,9 +61,6 @@ static void OPTIMIZE3 NORETURN stress_resched_child(
 	 *  "Normal" non-realtime scheduling policies
 	 */
 	static const int normal_policies[] = {
-#if defined(SCHED_OTHER)
-		SCHED_OTHER,
-#endif
 #if defined(SCHED_BATCH)
 		SCHED_BATCH,
 #endif
@@ -71,6 +69,9 @@ static void OPTIMIZE3 NORETURN stress_resched_child(
 #endif
 #if defined(SCHED_IDLE)
 		SCHED_IDLE,
+#endif
+#if defined(SCHED_OTHER)
+		SCHED_OTHER,
 #endif
 	};
 #endif
@@ -115,7 +116,7 @@ static void OPTIMIZE3 NORETURN stress_resched_child(
 #endif
 		}
 
-		VOID_RET(int, nice(1));
+		VOID_RET(int, shim_nice(1));
 
 		if (UNLIKELY(!stress_continue(args)))
 			break;
@@ -170,7 +171,8 @@ static int stress_resched(stress_args_t *args)
 	s_pids_max = max_prio + 1; /* 0.. max_prio */
 	s_pids = stress_sync_s_pids_mmap((size_t)s_pids_max);
 	if (s_pids == MAP_FAILED) {
-		pr_inf_skip("%s: failed to mmap %d PIDs, skipping stressor\n", args->name, s_pids_max);
+		pr_inf_skip("%s: failed to mmap %d PIDs%s, skipping stressor\n",
+			args->name, s_pids_max, stress_get_memfree_str());
 		return EXIT_NO_RESOURCE;
 	}
 	for (i = 0; i < s_pids_max; i++)
@@ -186,8 +188,10 @@ static int stress_resched(stress_args_t *args)
 				PROT_READ | PROT_WRITE,
 				MAP_ANONYMOUS | MAP_SHARED, -1, 0);
 	if (yields == MAP_FAILED) {
-		pr_inf_skip("%s: cannot mmap yield counter array, skipping stressor, errno=%d (%s)\n",
-			args->name, errno, strerror(errno));
+		pr_inf_skip("%s: failed to mmap %zu byte yield counter array%s, "
+			"errno=%d (%s), skipping stressor\n",
+			args->name, yields_size,
+			stress_get_memfree_str(), errno, strerror(errno));
 		rc = EXIT_NO_RESOURCE;
 		goto tidy_s_pids;
 	}
@@ -229,7 +233,7 @@ static int stress_resched(stress_args_t *args)
 	/*
 	 *  Dump stats for just instance 0 to reduce output
 	 */
-	if (args->instance == 0) {
+	if (stress_instance_zero(args)) {
 		uint64_t total_yields = 0;
 
 		for (i = 0; i < s_pids_max; i++)
@@ -264,7 +268,7 @@ tidy_s_pids:
 
 const stressor_info_t stress_resched_info = {
 	.stressor = stress_resched,
-	.class = CLASS_SCHEDULER | CLASS_OS,
+	.classifier = CLASS_SCHEDULER | CLASS_OS,
 	.verify = VERIFY_ALWAYS,
 	.help = help
 };
@@ -272,7 +276,7 @@ const stressor_info_t stress_resched_info = {
 #else
 const stressor_info_t stress_resched_info = {
 	.stressor = stress_unimplemented,
-	.class = CLASS_SCHEDULER | CLASS_OS,
+	.classifier = CLASS_SCHEDULER | CLASS_OS,
 	.verify = VERIFY_ALWAYS,
 	.help = help,
 	.unimplemented_reason = "built without Linux scheduling support"
