@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C) 2022-2024 Colin Ian King.
+ * Copyright (C) 2022-2025 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -18,6 +18,7 @@
  *
  */
 #include "stress-ng.h"
+#include "core-mmap.h"
 
 static const stress_help_t help[] = {
 	{ NULL,	"signest N",	 "start N workers generating nested signals" },
@@ -25,48 +26,100 @@ static const stress_help_t help[] = {
 	{ NULL,	NULL,		 NULL }
 };
 
+#if defined(HAVE_SIGLONGJMP)
+
+#define MAX_SIGNALS	(64)	/* Note: must NOT be larger than 8 * sizeof(signal_info.signalled) */
+
 static bool jmp_env_ok;
 static sigjmp_buf jmp_env;
 
-static const int signals[] = {
+static const int defined_signals[] = {
+#if defined(SIGABRT)
+	SIGABRT,
+#endif
+#if defined(SIGALRM)
+	SIGALRM,
+#endif
+#if defined(SIGBUS)
+	SIGBUS,
+#endif
+#if defined(SIGCHLD)
+	SIGCHLD,
+#endif
+#if defined(SIGCONT)
+	SIGCONT,
+#endif
+#if defined(SIGEMT)
+	SIGEMT,
+#endif
+#if defined(SIGFPE)
+	SIGFPE,
+#endif
 #if defined(SIGHUP)
 	SIGHUP,
 #endif
 #if defined(SIGILL)
 	SIGILL,
 #endif
-#if defined(SIGQUIT)
-	SIGQUIT,
+#if defined(SIGINFO)
+	SIGINFO,
 #endif
-#if defined(SIGABRT)
-	SIGABRT,
+#if defined(SIGINT)
+	SIGINT,
 #endif
-#if defined(SIGFPE)
-	SIGFPE,
-#endif
-#if defined(SIGTERM)
-	SIGTERM,
-#endif
-#if defined(SIGXCPU)
-	SIGXCPU,
-#endif
-#if defined(SIGXFSZ)
-	SIGXFSZ,
+#if defined(SIGIO)
+	SIGIO,
 #endif
 #if defined(SIGIOT)
 	SIGIOT,
 #endif
-#if defined(SIGSTKFLT)
-	SIGSTKFLT,
+#if defined(SIGLOST)
+	SIGLOST,
+#endif
+#if defined(SIGPIPE)
+	SIGPIPE,
+#endif
+#if defined(SIGPOLL)
+	SIGPOLL,
+#endif
+#if defined(SIGPROF)
+	SIGPROF,
 #endif
 #if defined(SIGPWR)
 	SIGPWR,
 #endif
-#if defined(SIGINFO)
-	SIGINFO,
+#if defined(SIGQUIT)
+	SIGQUIT,
 #endif
-#if defined(SIGVTALRM)
-	SIGVTALRM,
+#if defined(SIGSEGV)
+	SIGSEGV,
+#endif
+#if defined(SIGSTKFLT)
+	SIGSTKFLT,
+#endif
+#if defined(SIGSYS)
+	SIGSYS,
+#endif
+#if defined(SIGTERM)
+	SIGTERM,
+#endif
+#if defined(SIGTRAP)
+	SIGTRAP,
+#endif
+#if defined(SIGTSTP)
+	SIGTSTP,
+#endif
+#if defined(SIGTTIN)
+	SIGTTIN,
+#endif
+#if defined(SIGTTOU)
+	SIGTTOU,
+#endif
+#if defined(SIGUNUSED)
+	SIGUNUSED,
+#endif
+#if defined(SIGURG)
+	SIGURG,
 #endif
 #if defined(SIGUSR1)
 	SIGUSR1,
@@ -74,20 +127,30 @@ static const int signals[] = {
 #if defined(SIGUSR2)
 	SIGUSR2,
 #endif
-#if defined(SIGTTOU)
-	SIGTTOU,
-#endif
-#if defined(SIGTTIN)
-	SIGTTIN,
+#if defined(SIGVTALRM)
+	SIGVTALRM,
 #endif
 #if defined(SIGWINCH)
 	SIGWINCH,
 #endif
+#if defined(SIGXCPU)
+	SIGXCPU,
+#endif
+#if defined(SIGXFSZ)
+	SIGXFSZ,
+#endif
 };
 
+typedef struct {
+	int signum;		/* signal number */
+	bool signalled;		/* true = signal handled */
+} stress_signal_t;
+
+static stress_signal_t signals[MAX_SIGNALS] ALIGN64;
+static size_t max_signals;
+
 typedef struct {
 	stress_args_t *args;
-	uint32_t signalled;	/* bitmap of index into signals[] handled */
 	bool stop;		/* true to stop further nested signalling */
 	intptr_t altstack;	/* alternative stack push start */
 	intptr_t altstack_start;/* alternative start mmap start */
@@ -101,31 +164,21 @@ typedef struct {
 static volatile stress_signest_info_t signal_info;
 static uint64_t raised;
 static uint64_t handled;
+static size_t signal_index;
 
 static void stress_signest_ignore(void)
 {
 	size_t i;
 
-	for (i = 0; i < SIZEOF_ARRAY(signals); i++)
-		VOID_RET(int, stress_sighandler("signest", signals[i], SIG_IGN, NULL));
-}
-
-static inline ssize_t stress_signest_find(int signum)
-{
-	size_t i;
-
-	for (i = 0; i < SIZEOF_ARRAY(signals); i++) {
-		if (signals[i] == signum)
-			return (ssize_t)i;
-	}
-	return (ssize_t)-1;
+	for (i = 0; i < max_signals; i++)
+		VOID_RET(int, stress_sighandler("signest", signals[i].signum, SIG_IGN, NULL));
 }
 
 static void MLOCKED_TEXT stress_signest_handler(int signum)
 {
-	ssize_t i;
+	const int i = signum;
 	const intptr_t addr = (intptr_t)&i;
-	double run_time = stress_time_now() - signal_info.time_start;
+	const double run_time = stress_time_now() - signal_info.time_start;
 
 	handled++;
 	signal_info.depth++;
@@ -136,9 +189,8 @@ static void MLOCKED_TEXT stress_signest_handler(int signum)
 	/* using alternative signal stack? */
 	if ((addr >= signal_info.altstack_start) &&
 	    (addr < signal_info.altstack_end)) {
-		ptrdiff_t delta;
+		ptrdiff_t delta = signal_info.altstack - addr;
 
- 		delta = signal_info.altstack - addr;
 		if (delta < 0)
 			delta = -delta;
 		if (delta > signal_info.stack_depth)
@@ -147,14 +199,18 @@ static void MLOCKED_TEXT stress_signest_handler(int signum)
 
 	if (UNLIKELY(run_time > (double)g_opt_timeout)) {
 		stress_signest_ignore();
-		if (jmp_env_ok)
+		if (jmp_env_ok) {
 			siglongjmp(jmp_env, 1);
+			stress_no_return();
+		}
 	}
 
 	if (UNLIKELY(signal_info.stop)) {
 		stress_signest_ignore();
-		if (jmp_env_ok)
+		if (jmp_env_ok) {
 			siglongjmp(jmp_env, 1);
+			stress_no_return();
+		}
 	}
 
 	if (UNLIKELY(!signal_info.args))
@@ -163,34 +219,59 @@ static void MLOCKED_TEXT stress_signest_handler(int signum)
 	stress_bogo_inc(signal_info.args);
 	if (UNLIKELY(!stress_continue(signal_info.args))) {
 		stress_signest_ignore();
-		if (jmp_env_ok)
+		if (jmp_env_ok) {
 			siglongjmp(jmp_env, 1);
+			stress_no_return();
+		}
 	}
 
-	i = stress_signest_find(signum);
-	if (UNLIKELY((i < 0) || (i == (ssize_t)SIZEOF_ARRAY(signals))))
+	signals[signal_index].signalled = true;
+	signal_index++;
+	if (UNLIKELY(signal_index >= max_signals))
 		goto done;
 
-	signal_info.signalled |= 1U << i;
-
-	for (; i < (ssize_t)SIZEOF_ARRAY(signals); i++) {
-		if (signal_info.stop || !stress_continue(signal_info.args)) {
-			if (jmp_env_ok)
-				siglongjmp(jmp_env, 1);
+	if (signal_info.stop || !stress_continue(signal_info.args)) {
+		if (jmp_env_ok) {
+			siglongjmp(jmp_env, 1);
+			stress_no_return();
 		}
-		(void)shim_raise(signals[i]);
-		raised++;
 	}
-
+	(void)shim_raise(signals[signal_index].signum);
+	raised++;
 done:
 	--signal_info.depth;
 	return;
 }
 
+/*
+ *  stress_signest_shuffle()
+ *	randomly shuffle order of signals array
+ */
+static inline void stress_signest_shuffle(void)
+{
+	register size_t i;
+	
+	for (i = 0; i < max_signals; i++) {
+		register size_t j = (size_t)stress_mwc32modn((uint32_t)max_signals);
+		stress_signal_t tmp;
+
+		tmp = signals[i];
+		signals[i] = signals[j];
+		signals[j] = tmp;
+	}
+}
+
+static int stress_signest_cmp(const void *p1, const void *p2)
+{
+	const stress_signal_t *s1 = (const stress_signal_t *)p1;
+	const stress_signal_t *s2 = (const stress_signal_t *)p2;
+
+	return s1->signum - s2->signum;
+}
+
 /*
  *  stress_signest
- *	stress by generating segmentation faults by
- *	writing to a read only page
+ *	stress by raising next signal
  */
 static int stress_signest(stress_args_t *args)
 {
@@ -198,7 +279,7 @@ static int stress_signest(stress_args_t *args)
 	int n, ret, rc;
 	uint8_t *altstack;
 	char *buf, *ptr;
-	const size_t altstack_size = stress_get_min_sig_stack_size() * SIZEOF_ARRAY(signals);
+	const size_t altstack_size = stress_get_min_sig_stack_size() * MAX_SIGNALS;
 	double rate;
 	NOCLOBBER double t, duration;
 
@@ -206,15 +287,47 @@ static int stress_signest(stress_args_t *args)
 	handled = 0;
 	jmp_env_ok = false;
 
+	for (i = 0; i < SIZEOF_ARRAY(defined_signals); i++) {
+		signals[i].signum = defined_signals[i];
+		signals[i].signalled = false;
+	}
+	max_signals = i;
+#if defined(SIGRTMIN) && 	\
+    defined(SIGRTMAX)
+	{
+		int signum;
+
+		for (signum = SIGRTMIN; (signum <= SIGRTMAX) && (i < MAX_SIGNALS); i++, signum++) {
+			signals[i].signum = signum;
+			signals[i].signalled = false;
+		}
+		max_signals = i;
+	}
+#endif
+
+	/* Remove any duplicate signals */
+	qsort(signals, max_signals, sizeof(stress_signal_t), stress_signest_cmp);
+	for (i = 0; i < max_signals - 1; i++) {
+		if (signals[i].signum == signals[i + 1].signum) {
+			size_t j;
+
+			for (j = i; j < max_signals - 1; j++)
+				signals[j] = signals[j + 1];
+			max_signals--;
+		}
+	}
+
 	altstack = (uint8_t*)stress_mmap_populate(NULL, altstack_size,
 			PROT_READ | PROT_WRITE,
 			MAP_ANONYMOUS | MAP_PRIVATE, -1, 0);
 	if (altstack == MAP_FAILED) {
-		pr_inf_skip("%s: cannot allocate alternative signal stack, "
+		pr_inf_skip("%s: failed to mmap %zu byte alternative signal stack%s, "
 			"errno=%d (%s), skipping stressor\n",
-			args->name, errno, strerror(errno));
+			args->name, altstack_size, 
+			stress_get_memfree_str(), errno, strerror(errno));
 		return EXIT_NO_RESOURCE;
 	}
+	stress_set_vma_anon_name(altstack, altstack_size, "altstack");
 
 	if (stress_sigaltstack(altstack, altstack_size) < 0) {
 		(void)munmap((void *)altstack, altstack_size);
@@ -236,18 +349,24 @@ static int stress_signest(stress_args_t *args)
 		goto finish;
 	}
 
-	for (i = 0; i < SIZEOF_ARRAY(signals); i++) {
-		if (stress_sighandler(args->name, signals[i], stress_signest_handler, NULL) < 0)
+	for (i = 0; i < max_signals; i++) {
+		if (stress_sighandler(args->name, signals[i].signum, stress_signest_handler, NULL) < 0)
 			return EXIT_NO_RESOURCE;
 	}
 
 	jmp_env_ok = true;
+
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	t = stress_time_now();
 	do {
-		(void)shim_raise(signals[0]);
+		signal_index = 0;
+		(void)shim_raise(signals[signal_index].signum);
 		raised++;
+		if (UNLIKELY((raised & 0x3f) == 0))
+			stress_signest_shuffle();
 	} while (stress_continue(args));
 
 finish:
@@ -256,28 +375,29 @@ finish:
 	signal_info.stop = true;
 	stress_signest_ignore();
 
-	for (sz = 1, n = 0, i = 0; i < SIZEOF_ARRAY(signals); i++) {
-		if (signal_info.signalled & (1U << i)) {
-			const char *name = stress_get_signal_name(signals[i]);
+	for (sz = 1, n = 0, i = 0; i < max_signals; i++) {
+		if (signals[i].signalled) {
+			const char *name = stress_get_signal_name(signals[i].signum);
 
 			n++;
 			sz += name ? (strlen(name) + 1) : 32;
 		}
 	}
 
-	if (args->instance == 0) {
-		buf = calloc(sz, sizeof(*buf));
+	qsort(signals, n, sizeof(stress_signal_t), stress_signest_cmp);
+	if (stress_instance_zero(args)) {
+		buf = (char *)calloc(sz, sizeof(*buf));
 		if (buf) {
-			for (ptr = buf, i = 0; i < SIZEOF_ARRAY(signals); i++) {
-				if (signal_info.signalled & (1U << i)) {
-					const char *name = stress_get_signal_name(signals[i]);
+			for (ptr = buf, i = 0; i < max_signals; i++) {
+				if (signals[i].signalled) {
+					const char *name = stress_get_signal_name(signals[i].signum);
 
 					if (name) {
 						if (strncmp(name, "SIG", 3) == 0)
 							name += 3;
 						ptr += snprintf(ptr, (buf + sz - ptr), " %s", name);
 					} else {
-						ptr += snprintf(ptr, (buf + sz - ptr), " SIG%d", signals[i]);
+						ptr += snprintf(ptr, (buf + sz - ptr), " SIG%d", signals[i].signum);
 					}
 				}
 			}
@@ -305,7 +425,7 @@ finish:
 	}
 	rate = handled > 0 ? duration / (double)handled : 0.0;
 	stress_metrics_set(args, 0, "nanosec to handle a signal",
-		rate * 1000000000.0, STRESS_HARMONIC_MEAN);
+		rate * 1000000000.0, STRESS_METRIC_HARMONIC_MEAN);
 
 	stress_sigaltstack_disable();
 	(void)munmap((void *)altstack, altstack_size);
@@ -313,9 +433,21 @@ finish:
 	return rc;
 }
 
-stressor_info_t stress_signest_info = {
+const stressor_info_t stress_signest_info = {
 	.stressor = stress_signest,
-	.class = CLASS_SIGNAL | CLASS_OS,
+	.classifier = CLASS_SIGNAL | CLASS_OS,
 	.verify = VERIFY_ALWAYS,
 	.help = help
 };
+
+#else
+
+const stressor_info_t stress_signest_info = {
+	.stressor = stress_unimplemented,
+	.classifier = CLASS_SIGNAL | CLASS_OS,
+	.verify = VERIFY_ALWAYS,
+	.help = help,
+	.unimplemented_reason = "built without siglongjmp support"
+};
+
+#endif
