@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C)      2022 Colin Ian King.
+ * Copyright (C) 2022-2025 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -18,6 +18,9 @@
  *
  */
 #include "stress-ng.h"
+#include "core-builtin.h"
+#include "core-madvise.h"
+#include "core-mmap.h"
 
 #define MIN_SPLICE_BYTES	(1 * KB)
 #define MAX_SPLICE_BYTES	(64 * MB)
@@ -27,24 +30,14 @@
 
 static const stress_help_t help[] = {
 	{ NULL,	"splice N",	  "start N workers reading/writing using splice" },
-	{ NULL,	"splice-ops N",	  "stop after N bogo splice operations" },
 	{ NULL,	"splice-bytes N", "number of bytes to transfer per splice call" },
+	{ NULL,	"splice-ops N",	  "stop after N bogo splice operations" },
 	{ NULL,	NULL,		  NULL }
 };
 
-static int stress_set_splice_bytes(const char *opt)
-{
-	size_t splice_bytes;
-
-	splice_bytes = (size_t)stress_get_uint64_byte_memory(opt, 1);
-	stress_check_range_bytes("splice-bytes", splice_bytes,
-		MIN_SPLICE_BYTES, MAX_MEM_LIMIT);
-	return stress_set_setting("splice-bytes", TYPE_ID_SIZE_T, &splice_bytes);
-}
-
-static const stress_opt_set_func_t opt_set_funcs[] = {
-	{ OPT_splice_bytes,	stress_set_splice_bytes },
-	{ 0,			NULL }
+static const stress_opt_t opts[] = {
+	{ OPT_splice_bytes, "splice-bytes", TYPE_ID_SIZE_T_BYTES_VM, MIN_SPLICE_BYTES, MAX_MEM_LIMIT, NULL },
+	END_OPT,
 };
 
 #if defined(HAVE_SPLICE)
@@ -73,7 +66,7 @@ static unsigned int stress_splice_flag(void)
 static int stress_splice_pipe_size(const int fd)
 {
 #if defined(F_SETPIPE_SZ)
-	size_t pipe_size = (1 + (stress_mwc8() & 3)) * SPLICE_BUFFER_LEN;
+	const size_t pipe_size = (1 + (stress_mwc8() & 3)) * SPLICE_BUFFER_LEN;
 
 	return fcntl(fd, F_SETPIPE_SZ, pipe_size);
 #else
@@ -96,10 +89,10 @@ static inline int stress_splice_write(
 	ssize_t ret = 0;
 
 	while (size > 0) {
-		size_t n = (size_t)(size > buffer_len ? buffer_len : size);
+		const size_t n = (size_t)(size > buffer_len ? buffer_len : size);
 
 		ret = write(fd, buffer, n);
-		if (ret < 0)
+		if (UNLIKELY(ret < 0))
 			break;
 		size -= n;
 
@@ -114,15 +107,15 @@ static inline int stress_splice_write(
  */
 static bool stress_splice_non_block_write_4K(const int fd)
 {
-	char buffer[4096];
+	char buffer[4096] ALIGN64;
 	int flags;
 
 	flags = fcntl(fd, F_GETFL, 0);
-	if (flags < 0)
+	if (UNLIKELY(flags < 0))
 		return false;
-	if (fcntl(fd, F_SETFL, flags | O_NONBLOCK) < 0)
+	if (UNLIKELY(fcntl(fd, F_SETFL, flags | O_NONBLOCK) < 0))
 		return false;
-	(void)memset(buffer, 0xa5, sizeof(buffer));
+	(void)shim_memset(buffer, 0xa5, sizeof(buffer));
         if (write(fd, buffer, sizeof(buffer)) < 0)
 		return false;
         if (fcntl(fd, F_SETFL, flags) < 0)
@@ -142,16 +135,16 @@ static void stress_splice_looped_pipe(
 {
 	ssize_t ret;
 
-	if (!*use_splice_loop)
+	if (UNLIKELY(!*use_splice_loop))
 		return;
 
 	ret = splice(fds3[0], 0, fds4[1], 0, 4096, stress_splice_flag());
-	if (ret < 0) {
+	if (UNLIKELY(ret < 0)) {
 		*use_splice_loop = false;
 		return;
 	}
 	ret = splice(fds4[0], 0, fds3[1], 0, 4096, stress_splice_flag());
-	if (ret < 0) {
+	if (UNLIKELY(ret < 0)) {
 		*use_splice_loop = false;
 		return;
 	}
@@ -161,35 +154,43 @@ static void stress_splice_looped_pipe(
  *  stress_splice
  *	stress copying of /dev/zero to /dev/null
  */
-static int stress_splice(const stress_args_t *args)
+static int stress_splice(stress_args_t *args)
 {
 	int fd_in, fd_out, fds1[2], fds2[2], fds3[2], fds4[2];
-	size_t splice_bytes = DEFAULT_SPLICE_BYTES;
+	size_t splice_bytes, splice_bytes_total = DEFAULT_SPLICE_BYTES;
 	int rc = EXIT_FAILURE;
+	int metrics_count = 0;
 	bool use_splice = true;
 	bool use_splice_loop;
 	char *buffer;
 	ssize_t buffer_len;
+	double duration = 0.0, bytes = 0.0, rate;
 
-	if (!stress_get_setting("splice-bytes", &splice_bytes)) {
+	if (!stress_get_setting("splice-bytes", &splice_bytes_total)) {
 		if (g_opt_flags & OPT_FLAGS_MAXIMIZE)
-			splice_bytes = MAX_SPLICE_BYTES;
+			splice_bytes_total = MAX_SPLICE_BYTES;
 		if (g_opt_flags & OPT_FLAGS_MINIMIZE)
-			splice_bytes = MIN_SPLICE_BYTES;
+			splice_bytes_total = MIN_SPLICE_BYTES;
 	}
-	splice_bytes /= args->num_instances;
+	splice_bytes = splice_bytes_total / args->instances;
 	if (splice_bytes < MIN_SPLICE_BYTES)
 		splice_bytes = MIN_SPLICE_BYTES;
+	if (stress_instance_zero(args))
+		stress_usage_bytes(args, splice_bytes, splice_bytes * args->instances);
 
 	buffer_len = (ssize_t)(splice_bytes > SPLICE_BUFFER_LEN ?
 				SPLICE_BUFFER_LEN : splice_bytes);
-	buffer = mmap(NULL, (size_t)buffer_len, PROT_READ | PROT_WRITE,
+	buffer = stress_mmap_populate(NULL, (size_t)buffer_len,
+			PROT_READ | PROT_WRITE,
 			MAP_ANONYMOUS | MAP_PRIVATE, -1, 0);
 	if (buffer == MAP_FAILED) {
-		pr_inf("%s: cannot allocate write buffer, errno=%d (%s)\n",
-			args->name, errno, strerror(errno));
+		pr_inf("%s: failed to mmap %zu byte write buffer%s, errno=%d (%s)\n",
+			args->name, (size_t)buffer_len,
+			stress_get_memfree_str(), errno, strerror(errno));
 		goto close_done;
 	}
+	stress_set_vma_anon_name(buffer, buffer_len, "write-buffer");
+	(void)stress_madvise_mergeable(buffer, buffer_len);
 
 	if ((fd_in = open("/dev/zero", O_RDONLY)) < 0) {
 		pr_fail("%s: open /dev/zero failed, errno=%d (%s)\n",
@@ -248,6 +249,8 @@ static int stress_splice(const stress_args_t *args)
 	 */
 	use_splice_loop = stress_splice_non_block_write_4K(fds3[1]);
 
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	do {
@@ -263,9 +266,9 @@ static int stress_splice(const stress_args_t *args)
 		if (use_splice) {
 			ret = splice(fd_in, NULL, fds1[1], NULL,
 				splice_bytes, stress_splice_flag());
-			if (ret < 0) {
+			if (UNLIKELY(ret < 0)) {
 				if (errno == EINVAL) {
-					if (args->instance == 0) {
+					if (stress_instance_zero(args)) {
 						pr_inf("%s: using direct write to pipe and not splicing "
 							"from /dev/zero as this is not supported in "
 							"this kernel\n", args->name);
@@ -279,20 +282,42 @@ static int stress_splice(const stress_args_t *args)
 			ret = stress_splice_write(fds1[1], buffer,
 						  buffer_len,
 						  (ssize_t)splice_bytes);
-			if (ret < 0)
+			if (UNLIKELY(ret < 0))
 				break;
 		}
 
-		ret = splice(fds1[0], NULL, fds2[1], NULL,
-			splice_bytes, stress_splice_flag());
-		if (ret < 0)
-			break;
+		if (LIKELY(metrics_count++ < 1000)) {
+			/* fast non-metric path */
+			ret = splice(fds1[0], NULL, fds2[1], NULL,
+				splice_bytes, stress_splice_flag());
+			if (UNLIKELY(ret < 0))
+				break;
 
+			ret = splice(fds2[0], NULL, fd_out, NULL,
+				splice_bytes, stress_splice_flag());
+			if (UNLIKELY(ret < 0))
+				break;
+		} else {
+			/* slower metrics path */
+			double t;
 
-		ret = splice(fds2[0], NULL, fd_out, NULL,
-			splice_bytes, stress_splice_flag());
-		if (ret < 0)
-			break;
+			metrics_count = 0;
+			t = stress_time_now();
+			ret = splice(fds1[0], NULL, fds2[1], NULL,
+				splice_bytes, stress_splice_flag());
+			if (UNLIKELY(ret < 0))
+				break;
+			duration += stress_time_now() - t;
+			bytes += (double)ret;
+
+			t = stress_time_now();
+			ret = splice(fds2[0], NULL, fd_out, NULL,
+				splice_bytes, stress_splice_flag());
+			if (UNLIKELY(ret < 0))
+				break;
+			duration += stress_time_now() - t;
+			bytes += (double)ret;
+		}
 
 		/* Exercise -ESPIPE errors */
 		off_in = 1;
@@ -316,10 +341,6 @@ static int stress_splice(const stress_args_t *args)
 		VOID_RET(ssize_t, splice(fd_in, NULL, fds1[1], NULL,
 			1, ~0U));
 
-		/* Exercise 1 byte splice, zero flags */
-		VOID_RET(ssize_t, splice(fd_in, NULL, fds1[1], NULL,
-			1, 0));
-
 		/* Exercise splicing to oneself */
 		off_in = 0;
 		off_out = 0;
@@ -330,12 +351,17 @@ static int stress_splice(const stress_args_t *args)
 		stress_splice_looped_pipe(fds3, fds4, &use_splice_loop);
 		stress_splice_looped_pipe(fds3, fds4, &use_splice_loop);
 
-		inc_counter(args);
-	} while (keep_stressing(args));
+		stress_bogo_inc(args);
+	} while (stress_continue(args));
 
 	rc = EXIT_SUCCESS;
 
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
+
+	rate = (duration > 0.0) ? bytes / duration : 0.0;
+	stress_metrics_set(args, 0, "MB per sec splice rate",
+		rate / (double)MB, STRESS_METRIC_HARMONIC_MEAN);
+
 	(void)close(fd_out);
 close_fds4:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
@@ -364,17 +390,18 @@ close_done:
 	return rc;
 }
 
-stressor_info_t stress_splice_info = {
+const stressor_info_t stress_splice_info = {
 	.stressor = stress_splice,
-	.class = CLASS_PIPE_IO | CLASS_OS,
-	.opt_set_funcs = opt_set_funcs,
+	.classifier = CLASS_PIPE_IO | CLASS_OS,
+	.opts = opts,
 	.help = help
 };
 #else
-stressor_info_t stress_splice_info = {
-	.stressor = stress_not_implemented,
-	.class = CLASS_PIPE_IO | CLASS_OS,
-	.opt_set_funcs = opt_set_funcs,
-	.help = help
+const stressor_info_t stress_splice_info = {
+	.stressor = stress_unimplemented,
+	.classifier = CLASS_PIPE_IO | CLASS_OS,
+	.opts = opts,
+	.help = help,
+	.unimplemented_reason = "built without splice() system call"
 };
 #endif
