@@ -21,6 +21,7 @@
 #include "core-builtin.h"
 #include "core-cpu-cache.h"
 #include "core-mincore.h"
+#include "core-mmap.h"
 #include "core-out-of-memory.h"
 #include "core-pragma.h"
 #include "core-put.h"
@@ -35,8 +36,6 @@ typedef struct stress_stack_check {
 	struct stress_stack_check *self_addr;	/* Address of this struct to check */
 } stress_stack_check_t;
 
-static sigjmp_buf jmp_env;
-
 static const stress_help_t help[] = {
 	{ NULL,	"stack N",	"start N workers generating stack overflows" },
 	{ NULL,	"stack-fill",	"fill stack, touches all new pages " },
@@ -55,6 +54,10 @@ static const stress_opt_t opts[] = {
 	END_OPT,
 };
 
+#if defined(HAVE_SIGLONGJMP)
+
+static sigjmp_buf jmp_env;
+
 /*
  *  stress_segvhandler()
  *	SEGV handler
@@ -64,6 +67,7 @@ static void MLOCKED_TEXT NORETURN stress_segvhandler(int signum)
 	(void)signum;
 
 	siglongjmp(jmp_env, 1);		/* Ugly, bounce back */
+	stress_no_return();
 }
 
 /*
@@ -146,7 +150,7 @@ static bool OPTIMIZE3 stress_stack_alloc(
 		const uintptr_t page_mask = ~(uintptr_t)(page_size - 1);
 		const uintptr_t unmap_ptr = ((uintptr_t)&data[0] + (sizeof(data) >> 1)) & page_mask;
 
-		(void)stress_munmap_retry_enomem((void *)unmap_ptr, page_size);
+		(void)stress_munmap_force((void *)unmap_ptr, page_size);
 	}
 
 	/* traverse back down the stack to touch 128 pages on the stack */
@@ -209,7 +213,7 @@ static int stress_stack_child(stress_args_t *args, void *context)
 	}
 
 #if !defined(MADV_PAGEOUT)
-	if (stack_pageout && (args->instance == 0)) {
+	if (stack_pageout && (stress_instance_zero(args))) {
 		pr_inf("%s: stack-pageout not supported on this system\n", args->name);
 		stack_pageout = false;
 	}
@@ -225,9 +229,10 @@ static int stress_stack_child(stress_args_t *args, void *context)
 		PROT_READ | PROT_WRITE,
 		MAP_PRIVATE | MAP_ANONYMOUS, -1, 0);
 	if (altstack == MAP_FAILED) {
-		pr_inf_skip("%s: cannot allocate %zd byte signal stack: "
+		pr_inf_skip("%s: failed to mmap %zd byte signal stack%s, "
 			"errno=%d (%s), skipping stressor\n",
-			args->name, (size_t)STRESS_SIGSTKSZ, errno, strerror(errno));
+			args->name, (size_t)STRESS_SIGSTKSZ,
+			stress_get_memfree_str(), errno, strerror(errno));
 		return EXIT_NO_RESOURCE;
 	}
 	stress_set_vma_anon_name(altstack, STRESS_SIGSTKSZ, "altstack");
@@ -323,8 +328,21 @@ static int stress_stack(stress_args_t *args)
 
 const stressor_info_t stress_stack_info = {
 	.stressor = stress_stack,
-	.class = CLASS_VM | CLASS_MEMORY,
+	.classifier = CLASS_VM | CLASS_MEMORY,
 	.opts = opts,
 	.verify = VERIFY_ALWAYS,
 	.help = help
 };
+
+#else
+
+const stressor_info_t stress_stack_info = {
+	.stressor = stress_unimplemented,
+	.classifier = CLASS_VM | CLASS_MEMORY,
+	.opts = opts,
+	.verify = VERIFY_ALWAYS,
+	.help = help,
+	.unimplemented_reason = "built without siglongjmp support"
+};
+
+#endif
