@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C)      2022 Colin Ian King.
+ * Copyright (C) 2022-2025 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -20,6 +20,11 @@
 #include "stress-ng.h"
 #include "core-capabilities.h"
 
+#include <sys/ioctl.h>
+
+#if defined(HAVE_SYS_SELECT_H)
+#include <sys/select.h>
+#endif
 #if defined(HAVE_LINUX_RANDOM_H)
 #include <linux/random.h>
 #else
@@ -37,10 +42,13 @@ static const stress_help_t help[] = {
     defined(RNDADDTOENTCNT) ||	\
     defined(RNDRESEEDCRNG) || 	\
     defined(__linux__)
-static void check_eperm(const stress_args_t *args, const ssize_t ret, const int err)
+static void check_eperm(stress_args_t *args, const ssize_t ret, const int err)
 {
-	if ((g_opt_flags & OPT_FLAGS_VERIFY) &&
-	    ((ret == 0) || ((err != EPERM) && (err != EINVAL) && (err != ENOTTY)))) {
+	if (UNLIKELY((g_opt_flags & OPT_FLAGS_VERIFY) &&
+		     ((ret == 0) || ((err != EPERM) &&
+		      (err != ENOSYS) &&
+		      (err != EINVAL) &&
+		      (err != ENOTTY))))) {
 		pr_fail("%s: expected errno to be EPERM, got errno %d (%s) instead\n",
 			args->name, err, strerror(err));
 	}
@@ -51,7 +59,7 @@ static void check_eperm(const stress_args_t *args, const ssize_t ret, const int
  *  stress_urandom
  *	stress reading of /dev/urandom and /dev/random
  */
-static int stress_urandom(const stress_args_t *args)
+static int stress_urandom(stress_args_t *args)
 {
 	int fd_urnd, fd_rnd, fd_rnd_blk, rc = EXIT_FAILURE;
 #if defined(__linux__)
@@ -59,6 +67,7 @@ static int stress_urandom(const stress_args_t *args)
 #endif
 	bool sys_admin = stress_check_capability(SHIM_CAP_SYS_ADMIN);
 	double duration = 0.0, bytes = 0.0, rate;
+	const size_t page_size = args->page_size;
 
 	fd_urnd = open("/dev/urandom", O_RDONLY);
 	if (fd_urnd < 0) {
@@ -100,7 +109,7 @@ static int stress_urandom(const stress_args_t *args)
 #endif
 
 	if ((fd_urnd < 0) && (fd_rnd < 0)) {
-		if (args->instance == 0)
+		if (stress_instance_zero(args))
 			pr_inf_skip("%s: random device(s) do not exist, skipping stressor\n",
 				args->name);
 #if defined(__linux__)
@@ -112,6 +121,8 @@ static int stress_urandom(const stress_args_t *args)
 		return EXIT_NOT_IMPLEMENTED;
 	}
 
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	do {
@@ -127,7 +138,7 @@ static int stress_urandom(const stress_args_t *args)
 
 			t = stress_time_now();
 			ret = read(fd_urnd, buffer, sizeof(buffer));
-			if (ret >= 0) {
+			if (LIKELY(ret >= 0)) {
 				duration += stress_time_now() - t;
 				bytes += (double)ret;
 			} else {
@@ -146,9 +157,9 @@ static int stress_urandom(const stress_args_t *args)
 		if (fd_rnd >= 0) {
 			double t;
 #if defined(RNDGETENTCNT)
-			unsigned long val = 0;
+			unsigned long int val = 0;
 
-			if (ioctl(fd_rnd, RNDGETENTCNT, &val) < 0)
+			if (UNLIKELY(ioctl(fd_rnd, RNDGETENTCNT, &val) < 0))
 				goto next;
 			/* Try to avoid emptying entropy pool */
 			if (val < 128)
@@ -158,7 +169,7 @@ static int stress_urandom(const stress_args_t *args)
 #endif
 			t = stress_time_now();
 			ret = read(fd_rnd, buffer, 1);
-			if (ret >= 0) {
+			if (LIKELY(ret >= 0)) {
 				duration += stress_time_now() - t;
 				bytes += (double)ret;
 			} else {
@@ -235,10 +246,10 @@ next:
 		if (fd_urnd >= 0) {
 			void *ptr;
 
-			ptr = mmap(NULL, args->page_size, PROT_READ,
-				MAP_PRIVATE | MAP_ANONYMOUS, fd_urnd, 0);
+			ptr = mmap(NULL, page_size, PROT_READ,
+				MAP_PRIVATE, fd_urnd, 0);
 			if (ptr != MAP_FAILED)
-				(void)munmap(ptr, args->page_size);
+				(void)munmap(ptr, page_size);
 		}
 
 #if defined(HAVE_SELECT)
@@ -253,7 +264,7 @@ next:
 			FD_SET(fd_rnd_blk, &rdfds);
 
 			ret = select(fd_rnd_blk + 1, &rdfds, NULL, NULL, &timeout);
-			if (ret > 0) {
+			if (LIKELY(ret > 0)) {
 				if (FD_ISSET(fd_rnd_blk, &rdfds)) {
 #if defined(__linux__)
 					char *ptr;
@@ -267,20 +278,26 @@ next:
 					 *  single byte in the same way as reading /dev/zero does,
 					 *  as fixed in Linux kernel commit:
 					 *  "random: allow partial reads if later user copies fail"
+					 *
+					 *  mmap 2 pages, make second page read-only then write off
+					 *  the end of the first page.
 					 */
-					ptr = (char *)mmap(NULL, args->page_size, PROT_WRITE,
+					ptr = (char *)mmap(NULL, page_size * 2, PROT_WRITE,
 						MAP_PRIVATE | MAP_ANONYMOUS, -1, 0);
 					if (ptr != MAP_FAILED) {
-						double t;
-						/* Exercise 2 byte read on last 1 byte of page */
+						/* unmap last page.. */
+						if (mprotect((void *)(ptr + page_size), page_size, PROT_READ) == 0) {
+							double t;
 
-						t = stress_time_now();
-						ret = read(fd_rnd, ptr + args->page_size - 1, 2);
-						if (ret >= 0) {
-							duration += stress_time_now() - t;
-							bytes += (double)ret;
+							/* Exercise 2 byte read on last 1 byte of page */
+							t = stress_time_now();
+							ret = read(fd_rnd, ptr + args->page_size - 1, 2);
+							if (ret >= 0) {
+								duration += stress_time_now() - t;
+								bytes += (double)ret;
+							}
 						}
-						(void)munmap((void *)ptr, args->page_size);
+						(void)munmap((void *)ptr, page_size * 2);
 					}
 #endif
 					if (ret < 0) {
@@ -304,12 +321,14 @@ next:
 		}
 #endif
 
-		inc_counter(args);
-	} while (keep_stressing(args));
+		stress_bogo_inc(args);
+	} while (stress_continue(args));
 
-	stress_misc_stats_set(args->misc_stats, 0, "million random bits read", bytes * 8.0 / 1000000.0);
+	stress_metrics_set(args, 0, "million random bits read",
+		bytes * 8.0 / 1000000.0, STRESS_METRIC_GEOMETRIC_MEAN);
 	rate = (duration > 0.0) ? bytes * 8.0 / duration : 0.0;
-	stress_misc_stats_set(args->misc_stats, 1, "million random bits per sec", rate / 1000000.0);
+	stress_metrics_set(args, 1, "million random bits per sec",
+		rate / 1000000.0, STRESS_METRIC_HARMONIC_MEAN);
 
 	rc = EXIT_SUCCESS;
 err:
@@ -328,9 +347,9 @@ err:
 
 	return rc;
 }
-stressor_info_t stress_urandom_info = {
+const stressor_info_t stress_urandom_info = {
 	.stressor = stress_urandom,
-	.class = CLASS_DEV | CLASS_OS,
+	.classifier = CLASS_DEV | CLASS_OS,
 	.verify = VERIFY_OPTIONAL,
 	.help = help
 };
