@@ -20,6 +20,7 @@
 #include "stress-ng.h"
 #include "core-builtin.h"
 #include "core-killpid.h"
+#include "core-mmap.h"
 #include "core-sched.h"
 
 #include <sched.h>
@@ -51,6 +52,7 @@ static const stress_opt_t opts[] = {
      defined(__linux__)) &&			\
     (defined(SCHED_BATCH) ||			\
      defined(SCHED_DEADLINE) ||			\
+     defined(SCHED_EXT) ||			\
      defined(SCHED_IDLE) ||			\
      defined(SCHED_FIFO) ||			\
      defined(SCHED_OTHER) ||			\
@@ -85,7 +87,7 @@ static void stress_yield_sched_policy(stress_args_t *args, const size_t yield_sc
 		/*
 		 *  Only have 1 RT deadline instance running
 		 */
-		if (args->instance == 0) {
+		if (stress_instance_zero(args)) {
 			struct shim_sched_attr attr;
 
 			(void)shim_memset(&attr, 0, sizeof(attr));
@@ -102,25 +104,21 @@ static void stress_yield_sched_policy(stress_args_t *args, const size_t yield_sc
 			ret = shim_sched_setattr(0, &attr, 0);
 			break;
 		}
-		goto case_sched_other;
-#endif
-#if defined(SCHED_IDLE)
-	case SCHED_IDLE:
-		goto case_sched_other;
+		param.sched_priority = 0;
+		ret = sched_setscheduler(0, policy, &param);
+		break;
 #endif
 #if defined(SCHED_BATCH)
 	case SCHED_BATCH:
-		goto case_sched_other;
+#endif
+#if defined(SCHED_EXT)
+	case SCHED_EXT:
+#endif
+#if defined(SCHED_IDLE)
+	case SCHED_IDLE:
 #endif
 #if defined(SCHED_OTHER)
 	case SCHED_OTHER:
-#endif
-#if (defined(SCHED_DEADLINE) &&		\
-     defined(HAVE_SCHED_GETATTR) &&	\
-     defined(HAVE_SCHED_SETATTR)) ||	\
-     defined(SCHED_IDLE) ||		\
-     defined(SCHED_BATCH)
-case_sched_other:
 #endif
 		param.sched_priority = 0;
 		ret = sched_setscheduler(0, policy, &param);
@@ -254,8 +252,9 @@ static int stress_yield(stress_args_t *args)
 
 	pids = (pid_t *)calloc(yielders, sizeof(*pids));
 	if (!pids) {
-		pr_inf_skip("%s: calloc failed allocating %" PRIu32
-			" pids, skipping stressor\n", args->name, yielders);
+		pr_inf_skip("%s: failed to allocate %" PRIu32
+			" pids%s, skipping stressor\n",
+			args->name, yielders, stress_get_memfree_str());
 		return EXIT_NO_RESOURCE;
 	}
 
@@ -265,8 +264,9 @@ static int stress_yield(stress_args_t *args)
 			PROT_READ | PROT_WRITE,
 			MAP_SHARED | MAP_ANONYMOUS, -1, 0);
 	if (metrics == MAP_FAILED) {
-		pr_err("%s: mmap failed, count not allocate %zd bytes, errno=%d (%s)\n",
-			args->name, metrics_size, errno, strerror(errno));
+		pr_err("%s: failed to mmap %zu bytes%s, errno=%d (%s)\n",
+			args->name, metrics_size,
+			stress_get_memfree_str(), errno, strerror(errno));
 		free(pids);
 		return EXIT_NO_RESOURCE;
 	}
@@ -338,7 +338,7 @@ static int stress_yield(stress_args_t *args)
 
 const stressor_info_t stress_yield_info = {
 	.stressor = stress_yield,
-	.class = CLASS_SCHEDULER | CLASS_OS,
+	.classifier = CLASS_SCHEDULER | CLASS_OS,
 	.opts = opts,
 	.verify = VERIFY_OPTIONAL,
 	.help = help
@@ -346,7 +346,7 @@ const stressor_info_t stress_yield_info = {
 #else
 const stressor_info_t stress_yield_info = {
 	.stressor = stress_unimplemented,
-	.class = CLASS_SCHEDULER | CLASS_OS,
+	.classifier = CLASS_SCHEDULER | CLASS_OS,
 	.opts = opts,
 	.verify = VERIFY_OPTIONAL,
 	.help = help,
